#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include "mga_vid.h"

mga_vid_config_t mga_vid_config;
uint_8 *vid_data;

void display_frame(uint_8 *tab) {
  uint_8 *dest;
  uint_32 bespitch, h;
  
  dest = vid_data;
  bespitch = (mga_vid_config.src_width + 31) & ~31;

  for (h=0; h < mga_vid_config.src_height; h++) {
    memcpy (dest, tab, mga_vid_config.src_width);
    dest += bespitch;
  }
  for (h=0; h < mga_vid_config.src_height; h++) {
    memcpy (dest, tab, mga_vid_config.src_width/2);
    dest += bespitch/2;
  }
}

display_init(uint_32 width, uint_32 height)
{
	int f;

	f = open("/dev/mga_vid",O_RDWR);

	if(f == -1)
	{
		fprintf(stderr,"Couldn't open /dev/mga_vid\n");
		exit(1);
	}

	mga_vid_config.src_width = width;
	mga_vid_config.src_height= height;
	mga_vid_config.dest_width = width;
	mga_vid_config.dest_height= height;
	mga_vid_config.x_org= 10;
	mga_vid_config.y_org= 10;

	if (ioctl(f,MGA_VID_CONFIG,&mga_vid_config))
	{
		perror("Error in mga_vid_config ioctl");
	}
	ioctl(f,MGA_VID_ON,0);

	vid_data = (char*)mmap(0,256*4096,PROT_WRITE,MAP_SHARED,f,0);
}


int main(int argc,char *argv[])
{
	int i;
	int nb_frames;
	uint_8 *img;
	static struct timeval tv_beg;
	static struct timeval tv_end;
	float time;
	float bandwidth;

	img=(uint_8*)malloc(720);
	memset (img, 0, 720);
	
	display_init(720, 576);

	nb_frames = 300;
	fprintf(stderr, "sending %d video frames...", nb_frames);
	gettimeofday(&tv_beg, NULL);

	for (i=0; i<nb_frames; i++) {
		display_frame(img);
	}

	gettimeofday(&tv_end, NULL);
	fprintf(stderr, "done\n");

	time = (tv_end.tv_sec - tv_beg.tv_sec) +
			(tv_end.tv_usec - tv_beg.tv_usec)/1000000.0;
	bandwidth = ((double)nb_frames)*(720.0*576.0*3.0/2.0) / time;
	fprintf(stderr, "%d frames in %f sec.\nBandwidth = %f MBytes\n", nb_frames, time, bandwidth/1000000);

	return 0;
}

