/* sample application for the mgadvd driver */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include "mgadvd.h"

#define BLOCK_SIZE	2048

/* start of globals */

/* device names */
char *mgadvd_devname = "/dev/video1";
char *mgayuv_devname = "unused";

/* file descriptors */
int mgadvd_fd;
int mgayuv_fd;
int vob_fd;

/* lba offset of the VOB file */
unsigned long voblba = 0;

/* Stream IDs (video, audio and & sub-picture for DVD VOB type only) */
unsigned char vidsid = 0xE0;
unsigned char audsid = 0x80;
unsigned char spsid = 0x20;

/* device structures */
struct mgadvd_status_s summary;
struct video_play_mode pmode;

/* end of globals */

void usage(void)
{
	fprintf(stderr, "usage: vobcat <vob_file|-> [video sid] [audio sid] [subpic sid] [lba offset]\n");
}

void setup_video_window(void)
{
        struct video_window vw = {
                0, 0,
                1152, 864,
                0,
                0,
                NULL,
                0
        };
        
        if (ioctl(mgadvd_fd, VIDIOCSWIN, &vw) < 0)
                perror("configuring mgadvd window");
}

/* check for matching audio or subpic stream id's in the sector buffer
   return 1 if match, 0 if no match
   FIXME: need to add check for video stream id
*/
/* some shorthand */
#define STCODE(p,a,b,c,d) ((p)[0] == a && (p)[1] == b && (p)[2] == c && (p)[3] == d)
int stream_match(u_char *buf, u_char vid, u_char aid, u_char sid)
{
	u_char pesid = 0;

	/* look for program stream */
	if (!STCODE (buf,0x00,0x00,0x01,0xBA))
	{
		fprintf(stderr, "VOB is invalid (missing PS)\n");
		exit(0); /* remove this eventually with better logic */
	}

        /* look for packetized elementary stream */
	if (STCODE (buf+14,0x000,0x00,0x01,0xBD))
	{
		pesid = buf[23 + buf[22]];
		if ( (pesid != aid) && (pesid != sid) )
			return 0;
	}
	return 1;
}

void send_video_stream(void)
{
	int read_ok;
	int count;
	unsigned char sector[BLOCK_SIZE];

	count = 0;

	/* vob_fd already open from parse_args */
        do
        {
          read_ok = read( vob_fd, sector, BLOCK_SIZE );
          /* skip sectors that dont match stream id's we want */ 
          if ( !stream_match(sector, vidsid, audsid, spsid) )
                 continue;
          fprintf(stderr,"%8lu\015",count + voblba);
	  count++;
          if ( write(mgadvd_fd, sector, BLOCK_SIZE) < 0)
                 perror("writing bitstream");
        } while (read_ok > 0);

        fprintf(stderr,"\n-- Fini --\n");

	close(mgadvd_fd);
}

void open_mgadvd(void)
{
	mgadvd_fd = open(mgadvd_devname, O_RDWR);
	if (mgadvd_fd < 0)
	{
		perror("error opening DVD decoder for read/write");
		exit(1);
	}
}

void set_streams(void)
{
	/* default to DVD microcode, AC3 microcode, stream type DVD VOB */
	if (ioctl(mgadvd_fd, I34_IOCSSTREAMTYPE, 0x1102) < 0)
		perror("setting microcode/bitstream types");

	if (ioctl(mgadvd_fd, I34_IOCSVIDEOSID, vidsid) < 0)
		perror("setting Video Stream ID");

	if (ioctl(mgadvd_fd, I34_IOCSAUDIOSID, audsid) < 0)
			perror("setting Audio Stream ID");

	if (ioctl(mgadvd_fd, I34_IOCSSUBPICSID, spsid) < 0)
			perror("setting Sub-picture Stream ID");
}

void parse_args(int argc, char *argv[])
{
	/* if a filename isn't given, there is nothing to do, show usage */
	if (argc < 2) {
		usage();
		exit(1);
	}
	/* check for filename of '-' to indicate using stdin instead of file */
	if (strcmp(argv[1], "-") == 0)
	{
		vob_fd = 0;
	} else {
			if ((vob_fd = open(argv[1], O_RDONLY)) <= 0)
			{
				perror("unable to open DVD VOB stream");
				exit(1);
			}
	}

	if (argc > 2) vidsid |= atoi(argv[2]) & 15;
		
	if (argc > 3) audsid |= atoi(argv[3]) & 15;

	if (argc > 4) spsid  |= atoi(argv[4]) & 31;
		
	/* only allow seek forward on file, not on stdin
	   FIXME: add read and toss somewhere for stdin */

	if ( (argc > 5) && (vob_fd > 0) )
	{
		voblba = atol(argv[5]);
		fprintf(stderr, "seeking to lba %lu (byte %lu)\n",
			voblba, lseek(vob_fd, voblba << 11, SEEK_SET) );
	}
	/* show the options or the defaults */
	fprintf(stderr, "options: VID %02X, AUD %02X, SP %02X, LBA %06lX\n",
			vidsid, audsid, spsid, voblba);
}

int main(int argc, char *argv[])
{
	/* open the file if it's given, and parse any other options */
	parse_args(argc, argv);

	/* open the decoder device, should be /dev/video1 */
	open_mgadvd();

	fprintf(stderr,"Matrox/Zoran DVD device is %s\n",mgadvd_devname);

	setup_video_window();

	/* set DVD VOB bitstream type, stream ID options or defaults
	   since this communicates via ioctl to mgadvd device, that
	   device MUST be open already */

	set_streams();

	pmode.mode = VID_PLAY_NORMAL;
	if (ioctl(mgadvd_fd, VIDIOCSPLAYMODE, &pmode) < 0) {
		perror("starting codec");
		exit(5);
	}

        if (ioctl(mgadvd_fd, VIDIOCCAPTURE, 1) < 0)
                perror("enabling video preview");

	send_video_stream();

	/* newline so we don't overwrite our LBA counter */
	fprintf(stderr, "\n");

	close(vob_fd);
	close(mgadvd_fd);

	return 0;
}









