/*
 * Video Capture Driver ( Video for Linux 1/2 )
 * for the Matrox Marvel G200 and Rainbow Runner-G series
 *
 * This module is an interface to the G100 and G200 video extension
 * registers.  
 *
 * Copyright (C) 1999  Ryan Drake <stiletto@mediaone.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef MGAVIDEO_H
#define MGAVIDEO_H

#define MAJOR_VERSION 0 
#define MINOR_VERSION 1

/* video registers */
#define VINCTL0         0x3e00
#define VINCTL1         0x3e04
#define VBIADDR0        0x3e08
#define VBIADDR1        0x3e0c
#define VINADDR0        0x3e10
#define VINADDR1        0x3e14
#define VINNEXTWIN      0x3e18
#define VBICOUNT	0x3e18	/* G400 */
#define VINCTL          0x3e1c
#define VINADDR2	0x3e20	/* G400 */
#define VINADDR3	0x3e24	/* G400 */
#define VINCTL2		0x3e28	/* G400 */
#define VINCTL3		0x3e2c	/* G400 */
#define VSTATUS         0x3e30
#define VICLEAR         0x3e34
#define VIEN            0x3e38
#define VINHEIGHT	0x3e3c	/* G400 */

/* CODEC registers */
#define CODECCTL        0x3e40
#define CODECMISCCTL	0x3e43
#define CODECADDR       0x3e44
#define CODECHOSTPTR    0x3e48
#define CODECHARDPTR    0x3e4c
#define CODECLCODE      0x3e50
#define RST             0x1e40  /* not a codec register, but it helps though... */

/* Backend scaler registers */
#define BESA1ORG        0x3d00
#define BESA2ORG        0x3d04
#define BESB1ORG        0x3d08
#define BESB2ORG        0x3d0c
#define BESA1CORG       0x3d10
#define BESA2CORG       0x3d14
#define BESB1CORG       0x3d18
#define BESB2CORG       0x3d1c
#define BESCTL          0x3d20
#define BESPITCH        0x3d24
#define BESHCOORD       0x3d28
#define BESVCOORD       0x3d2c
#define BESHISCAL       0x3d30
#define BESVISCAL       0x3d34
#define BESHSRCST       0x3d38
#define BESHSRCEND      0x3d3c
#define BESLUMACTL	0x3d40	/* G400 */
#define BESV1WGHT       0x3d48
#define BESV2WGHT       0x3d4c
#define BESHSRCLST      0x3d50
#define BESV1SRCLST     0x3d54
#define BESV2SRCLST     0x3d58
#define BESA1C3ORG	0x3d60	/* G400 */
#define BESA2C3ORG	0x3d64	/* G400 */
#define BESB1C3ORG	0x3d68	/* G400 */
#define BESB2C3ORG	0x3d6c	/* G400 */
#define BESGLOBCTL      0x3dc0
#define BESSTATUS       0x3dc4

#define VINCTLx_MASK    0x00000fff
#define VBIADDRx_MASK   0x00fffE00
#define VINADDRx_MASK   0x00fffE00

/* This defines which field comes first (top or bottom)
   (to select bottom field put VS_FRISTFIELD = VS_FIELD)
 */
#define	VS_FIRSTFIELD	0

/* mga video status */
#define VS_IVSYNC    0x0001 /* input vsync interrupt */
#define VS_CMDCMPL   0x0002 /* command complete interrupt */
#define VS_BLVL      0x0004 /* buffer level interrupt */
#define VS_EOI       0x0008 /* codec end of image interrupt */
#define VS_FIELD     0x0100 /* field 0 or 1 detected */
#define VS_VIDEO     0x0200 /* indicates video was captured */
#define VS_RAWVBI    0x0400 /* indicates raw vbi was captured */
#define VS_SLICEDVBI 0x0800 /* indicates sliced vbi was captured */
#define VS_STALLED   0x1000 /* codec stalled after transfer stopped */
#define VS_TASKDETD  0x2000 /* video input task detected, G400 only */
#define VS_GRABPTR   0xb000 /* pointer to the last field grabbed, G400 only */

#define VI_IVSYNC    0x0001 /* input vsync interrupt */
#define VI_CMDCMPL   0x0002 /* command complete interrupt */
#define VI_BLVL      0x0004 /* buffer level interrupt */
#define VI_EOI       0x0008 /* codec end of image interrupt */
#define VI_STALLED   0x1000 /* codec stalled after transfer stopped */

#define	VC_CODECEN	0x0001	/* enable codec interface */
#define	VC_MODE		0x0002	/* codec mode VMI/I33 */
#define	VC_TRIG		0x0004	/* codec command trigger */
#define	VC_DATAIN	0x0008	/* Decompress/Compress */
#define	VC_VMIMODE	0x0010	/* VMI mode A/B */
#define	VC_STOP		0x0020	/* Stop after current field */
#define	VC_TRANSEN	0x0040	/* enable transfer to/from frame buffer */


#define VIN_OFF      0x0
#define VIN_ON       0x1
#define VBI_OFF      0x0
#define VBI_RAW      0x2
#define VBI_SLICED   0x6


/* video attributes - these should probably be configurable on the fly
 *                    so users with different desktop sizes can keep
 *                    captured data off the desktop
 */
#define _VINWID   704
#define _VINHGT   _VINHGT_PAL
#define _VINHGT_NTSC   240
#define _VINHGT_PAL    288

#define	_VIN_WINDOW	(704 * 291 * 2)

#define	_VBI_WINDOW	(704 * 64 * 2)

/* the JPEG frame buffer size allocated in the MGA memory is called CODEC_BUFFER_SIZE */
/* This is the user app interaction buffer */
#define JPEG_BUFFSIZE 256*1024

/* values for mga_type field. */

#define MGA_G200	1
#define MGA_G400	2
#define MGA_G100	4

/* VBICOUNT values */

#define VBI1XNTSC	0x0b140a13
#define VBI2XNTSC	0x00140014
#define VBI1XPAL	0x11181119
#define VBI2XPAL	0x00190019

/****************************************************************************
* mga_dev : represents one mga board.  since we can be dealing with multiple
*           boards, all entry points to this module must specify a board
****************************************************************************/

struct mga_dev
{
        struct i2c_bus i2c;

        u8* fb;
        u8* ctrl;

        int inuse;
        char name[16];
	int mga_type;

	int	tv_option;	/* addon card or marvel */

        /* Device info from pci */
        u32 device;
        int irq;  
	int irq_status;
	int interrupt_lock;
	int interrupt_mask;

        /* caps */
        int hasaudio;
        int hastuner;
        int hasdecoder;
        int hasmjpeg;
        int hastvout;
  
        /* status of the video grabber */
        u32 status;
	u32 buffer_status[4];
	u32 buffer_flags[4];
	u32 buffer_stamp[4];
	u32 vin_active;		/* current active VIN window (0 or 1) (g200) */
	u32 active_buffer;	/* buffer being grabbed (0..3) */
	u32 next_active_buffer; /* next buffer being grabbed (0..3) */
	u32 field_req;
	u32 frame_next;

#define	FIELD_1			0x0001
#define	FIELD_2			0x0002
#define	FIELD_3			0x0004
#define	FIELD_4			0x0008
#define FIELD_INTERLACED	0x1000
#define	BUFFLAG_CLIENTLOCK	0x01
#define BUFFLAG_SERVERLOCK	0x02
#define	BUFFLAG_FIELDTYPE	0x04

#define	FRAME_1			0x0001
#define	FRAME_2			0x0002

        /* registered bottom half of isr */
        struct tq_struct tqnode_dpc;

        /* address for even and odd frames */
        u32 	vin_addr[4];
        u32 	vbi_addr[4];
	u32	vinpitch;
	u32	vinctl_value;

        u32	codecaddr; /* The JPEG buffers address in the MGA, seen from the CPU (aka interface_base)*/
        u32	codeccmd;  /* address of commando buffer for the MGA CODEC 
			      transfer engine (CTE) - (aka command_base) */
	u32	codecdata;      /* address of read data buffer for the MGA CTE - (aka read_base) */
	u32	host_interface_base; /* JPEG buffers address, seen from the MGA */
	u32	codec_offset;

	u32	codec_head;
	u32	codec_tail;
	enum	buz_codec_mode mjpeg_codec_mode;
        u32	lcode; /* length of the JPEG ? */
        
	u8 *interface_base;
        u8 *command_base;
        u8 *read_base;


	u32 codecctl_shadow;
	u32 codecmisc_shadow;

        u32 streamindex;
	int transfer;		/* is a bitstream transfer taking place ? */
        int grabbed;            /* the number of grabbed, uncompressed V4L frames */
   
	u32 cmdbuffer[64];
	int cmdindex;
	u16 databuffer[128];
	int dataindex;

	/* Information about the framebuffer layout (used for BES configuration) */
	u32 desktop_depth;	/* Depth : 8, 16, 24 or 32 */
	u32 desktop_width;      /* This is the real width (not virtual) */
	u32 desktop_height;     /* This is the real height */
	u32 desktop_offsetx;	/* X, Y offset of the displayed area */
	u32 desktop_offsety;	/* of the virtual desktop */

	/* Parameters for the BES : might be user defined */
	u32 bes_crop_top;       /* These are positive values, giving the number */
	u32 bes_crop_bottom;    /* of pixels to skip. */
	u32 bes_crop_left;
	u32 bes_crop_right;
	u32 video_activelines;  /* Height of the video fields (depends on PAL or NTSC)*/
	int interlaced_preview;

	/* These ones are computed by mgavideo_set_window */
	u32 besctl_value;
	u32 bes_origin_topfield;
	u32 bes_origin_bottomfield;


        struct	zr36060_dev	zr36060; /* holds the parameters for the ZR36060 */

        int preview_active;     /* shows if the preview is active in the moment */
        int memgrab_active; /* shows if memory grabbing is running */

	spinlock_t lock;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0))
	struct wait_queue *write_queue;
#else
	wait_queue_head_t write_queue;
#endif
};

struct mga_dev;

/* component device control */
void mgavideo_tuner( struct mga_dev* mga, unsigned int cmd, void* arg );
void mgavideo_audio( struct mga_dev* mga, unsigned int cmd, void* arg );
void mgavideo_decoder( struct mga_dev* mga, unsigned int cmd, void* arg );
int mgavideo_zr36060_attach(struct mga_dev *mga);

#define MGAVIDEO_HAS_TUNER   1
#define MGAVIDEO_HAS_AUDIO   2
#define MGAVIDEO_HAS_DECODER 4
#define MGAVIDEO_HAS_MJPEG   8
#define MGAVIDEO_HAS_TVOUT   16
int mgavideo_querycaps( struct mga_dev* mga );

/* get pointer to currently grabbed video */
void mgavideo_lock_video( struct mga_dev* mga, int *stamp, int *field, u8 **buffer);
void mgavideo_unlock_video( struct mga_dev* mga, int *stamp, int flag);

/* video interrupt control */
void mgavideo_command_enable(struct mga_dev* mga, int en);
void mgavideo_buffer_enable( struct mga_dev* mga, int en );
void mgavideo_eoi_enable( struct mga_dev* mga, int en );
void mgavideo_ivsync_enable( struct mga_dev* mga, int en );
void mgavideo_register_bh( struct mga_dev* mga, void(*routine)(void*), void* data );
void  mgavideo_set_dims(struct mga_dev *mga, int x, int y, int flags);

/* video preview */
void* mgavideo_get_base( struct mga_dev* mga );
void mgavideo_set_window( struct mga_dev* mga, int x, int y, int w, int h );
void mgavideo_set_colorkey( struct mga_dev* mga, int r, int g, int b );
void mgavideo_set_overlay( struct mga_dev* mga );
void mgavideo_preview_enable( struct mga_dev* mga, int en );
                                
/* find next mga adapter */
struct mga_dev* mgavideo_get( void );

/* release a mga adapter that is not needed anymore */
void mgavideo_release( struct mga_dev* mga );

/* If we are using an "old" kernel */
#ifndef PCI_DEVICE_ID_MATROX_G400
#define PCI_DEVICE_ID_MATROX_G400 0x0525
#endif

#endif /* MGAVIDEO_H */

