/***************************************************************************
                          objects.cpp  -  description
                             -------------------
    begin                : Wed Dec 8 1999
    copyright            : (C) 1999 by Jeff Anderson
    email                : jbanders@mines.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef OBJECTS_H
# define OBJECTS_H
#include "objects.h"
#endif

#ifndef EXTENSIONS_H
# define EXTENSIONS_H
#include "extensions.h"
#endif

#ifndef BUFFER_H
# define BUFFER_H
#include "buffer.h"
#endif

#ifndef STATS_H
# define STATS_H
#include "stats.h"
#endif

#ifndef VLC_H
#define VLC_H
#include "vlc.h"
#endif

#include <assert.h>

//FIXME
extern STATS mpstat;
extern OBUFF outbuff;
extern IBUFF input;

static BYTE_t zig_zag_scan[8][8] =
{
	{0,1,5,6,14,15,27,28},
	{2,4,7,13,16,26,29,42},
	{3,8,12,17,25,30,41,43},
	{9,11,18,24,31,40,44,53},
	{10,19,23,32,30,45,52,54},
	{20,22,33,38,46,51,55,60},
	{21,34,37,47,50,56,59,61},
	{35,36,48,49,57,58,62,63}
};

static BYTE_t alt_scan[8][8] =
{
	{0,4,6,20,22,36,38,52},
	{1,5,7,21,23,37,39,53},
	{2,8,19,24,34,40,50,54},
	{3,9,18,25,35,41,51,55},
	{10,17,26,30,42,46,56,60},
	{11,16,27,31,43,47,57,61},
	{12,15,28,32,44,48,58,62},
	{13,14,29,33,45,49,59,62}
};

/*****************************************************************************/
void BLOCK::set(const BYTE_t colour_comp,const blk_t value)
{
	cc = colour_comp;

	block_data = value;
} // end set()

void BLOCK::code(void)
{
	dct(block_data, coded_data);
	quantize_intra(coded_data);
	scan( mpstat.get_scan_type() );
} // end code()

void BLOCK::scan(const bool scan_order)
{
	if(scan_order)
	{
		for(BYTE_t i = 0; i < 8; i++)
		{
			for(BYTE_t j = 0; j < 8; j++)
			{
				scan_data[alt_scan[j][i]] = coded_data[i][j];
			}//end for
		}//end for
	}//end if
	else
	{
		for(int i = 0; i < 8; i++)
		{
			for(int j = 0; j < 8; j++)
			{
				scan_data[zig_zag_scan[j][i]] = coded_data[i][j];
			}//end for
		}//end for
	}//end else
} // end scan()

void BLOCK::write_block(short *predictor)
{
	put_intra_block(cc, predictor, scan_data);
} // end write_block()

BLOCK::~BLOCK()
{
} //end ~BLOCK()
/*****************************************************************************/
MACROBLOCK::MACROBLOCK()
{
	Y_block = new BLOCK [4];
	//FIXME
	// needs changed to work on video other than 4:2:0
	Cr_block = new BLOCK;
	Cb_block = new BLOCK;
}

//FIXME fix this function
void MACROBLOCK::set(const bool first_mb_in_slice, const u16 mb_row, const u16 mb_width, const u16 address,
										 const blk_t Y, const blk_t Cr, const blk_t Cb)
{
	macroblock_address = address;

	//FIXME may need work
	macroblock_address_increment = 1;

	next_Y_block = new blk_t[4];

	Y_block[0].set(0, Y);

	next_Y_block[1] = new blk_row_t [8];
	next_Y_block[2] = new blk_row_t [8];
	next_Y_block[3] = new blk_row_t [8];

	for(int x = 0; x < 8; x++)
	{
		next_Y_block[1][x] = Y[x+8];
	}
	Y_block[1].set(0, next_Y_block[1]);

	for(int x = 0; x < 8; x++)
	{
		next_Y_block[2][x] = &Y[x][8];
	}
	Y_block[2].set(0, next_Y_block[2]);

	for(int x = 0; x < 8; x++)
	{
		next_Y_block[3][x] = &Y[x+8][8];
	}
	Y_block[3].set(0, next_Y_block[3]);

	//FIXME
	// needs changed to work on video other than 4:2:0
	Cr_block->set(2, Cr);
	Cb_block->set(1, Cb);

} // end set()

void MACROBLOCK::code(void)
{
	for(int i = 0; i < 4; i++)
	{
		Y_block[i].code();
	}

	Cb_block->code();
	Cr_block->code();
} //end code()

void MACROBLOCK::write_macroblock(short *dct_predictor)
{
	putVLC_address_increment(macroblock_address_increment);
	//FIXME
	putVLC_macroblock_type();

	for(int i = 0; i < 4; i++)
	{
		Y_block[i].write_block(dct_predictor);
	}

	Cb_block->write_block(dct_predictor);
	Cr_block->write_block(dct_predictor);
} // end write_macroblock()

MACROBLOCK::~MACROBLOCK()
{
	delete [] Y_block;
	//FIXME
	// needs changed to work on video other than 4:2:0
	delete Cr_block;
	delete Cb_block;

	delete [] next_Y_block[1];
	delete [] next_Y_block[2];
	delete [] next_Y_block[3];

	delete [] next_Y_block;
} //end ~MACROBLOCK()
/*****************************************************************************/
SLICE::SLICE()
{
	length = mpstat.get_slice_length();

	//FIXME
	dct_predictor[0] = 0;
	dct_predictor[1] = 0;
	dct_predictor[2] = 0;

	//FIXME
	intra_slice_flag = 0;
	extra_bit_slice = 0;
  quantiser_scale_code = 1;

	mblock = new MACROBLOCK[length];
} // end SLICE()

void SLICE::set(const u16 slice_row,const blk_t Y,const blk_t Cr,const blk_t Cb)
{
	bool first_mb;
	slice_start_code = slice_row + 1;
	row = slice_row;

	for(u32 i = 0; i < length; i++)
	{
		if(i == 0)
			first_mb = (1==1);
		else
			first_mb = (1==0);

		mblock[i].set(first_mb, slice_row+1, length, ((row*length)+i), Y+(i*16), Cr+(i*8), Cb+(i*8) );
	} // end for
} //end set()

void SLICE::code()
{
	for(u32 i = 0; i < length; i++)
	{
		mblock[i].code();
	}
} //end code()

void SLICE::write_slice(void)
{
	outbuff.byte_allign();

	outbuff.put_bits(1,24);	//start code
	outbuff.put_bits(slice_start_code, 8);
	outbuff.put_bits(quantiser_scale_code, 5);
	outbuff.put_bits(intra_slice_flag, 1);

	for(u32 i = 0; i < length; i++)
	{
		mblock[i].write_macroblock(dct_predictor);
	}	// end for

} // end write_slice()

SLICE::~SLICE()
{
 	delete [] mblock;
}
/*****************************************************************************/
PICTURE::PICTURE(BYTE_t picture_type, u16 temporal_reference_value, STATS &mpstat)
{
 	temporal_reference = temporal_reference_value;
	picture_coding_type = picture_type;

	vbv_delay = mpstat.get_vbv_delay();

	horizontal_size = mpstat.get_horizontal_size();
	vertical_size = mpstat.get_vertical_size();
	
	color_horizontal_size = mpstat.get_color_horizontal_size();
	color_vertical_size = mpstat.get_color_vertical_size();

	slice_count = mpstat.get_numb_slice();

	Y_frame = Cr_frame = Cb_frame = NULL;

	slice = new SLICE[slice_count];
} //end PICTURE()

void PICTURE::read_picture()
{
// allocate memory for frame
//create an array of Y_data[hrz_size][vert_size]
	Y_frame = new blk_row_t [horizontal_size];
	if(Y_frame == NULL)
		exit(1);

	Cr_frame = new blk_row_t [color_horizontal_size];
	Cb_frame = new blk_row_t [color_horizontal_size];

	for(int i = 0; i < horizontal_size; i++)
	{
		Y_frame[i] = NULL;
		Y_frame[i] = new blk_element_t [vertical_size];
		if(Y_frame[i] == NULL)
			exit(1);
	}

	for(int i = 0; i < color_horizontal_size; i++)
	{
		Cr_frame[i] = Cb_frame[i] = NULL;
		Cr_frame[i] = new blk_element_t [color_vertical_size];
		Cb_frame[i] = new blk_element_t [color_vertical_size];
		if(Cr_frame[i] == NULL)
		{
			cout << "CR NULL" << endl;
			exit(1);
		} // end if
		if(Cb_frame[i] == NULL)
		{
			cout << "CB NULL" << endl;
			exit(1);
		} // end if
	} // end for

	//use ibuff to get next frame
	input.get_frame(Y_frame, Cr_frame, Cb_frame, mpstat);

	y_slice = new blk_t [slice_count];
	cr_slice = new blk_t [slice_count];
	cb_slice = new blk_t [slice_count];


	for(u16 i = 0; i < (slice_count); i++)
	{
		y_slice[i] = new blk_row_t [horizontal_size];
		cr_slice[i] = new blk_row_t [horizontal_size/2];
		cb_slice[i] = new blk_row_t [horizontal_size/2];

		for(int x = 0; x < horizontal_size; x++)
			y_slice[i][x] = &Y_frame[x][i*16];

		for(int x = 0; x < (horizontal_size/2); x++)
		{
			cr_slice[i][x] = &Cr_frame[x][i*8];
			cb_slice[i][x] = &Cb_frame[x][i*8];
		}
	
		// FIXME only works for 4:2:0
		slice[i].set( i, y_slice[i], cr_slice[i], cb_slice[i] );
	} //end for
} //end read_picture()

void PICTURE::code_picture(void)
{
	//FIXME may move to better performance
	for(int i = 0; i < slice_count; i++)
	{
		slice[i].code();
	}
}	//end code_picture()

void PICTURE::write_picture(void)
{
	outbuff.put_bits(0,8);
	outbuff.byte_allign();

	outbuff.put_bits(1,24);	//start code
	outbuff.put_bits(picture_start_code, 8);
	outbuff.put_bits(temporal_reference, 10);
	outbuff.put_bits(picture_coding_type, 3);
	outbuff.put_bits(vbv_delay, 16);
	outbuff.put_bits(extra_bit_picture, 1);

	pic_code_ext.write();

	//FIXME may move to better performance
	for(int i = 0; i < slice_count; i++)
	{
		slice[i].write_slice();
	} //end for

} //end write_picture()

PICTURE::~PICTURE()
{
	delete [] slice;
	slice = NULL;

	for(u16 i = 0; i < (slice_count); i++)
	{
		delete [] y_slice[i];
		delete [] cr_slice[i];
		delete [] cb_slice[i];
	}

	delete [] y_slice;
	delete [] cr_slice;
	delete [] cb_slice;


	for(int i = 0; i < horizontal_size; i++)
	{
		delete [] Y_frame[i];
	}

	for(int i = 0; i < color_horizontal_size; i++)
	{
		delete [] Cr_frame[i];
		delete [] Cb_frame[i];
	} // end for

	delete [] Y_frame;
	delete [] Cr_frame;
	delete [] Cb_frame;
}
/*****************************************************************************/
GOP::GOP(STATS &mpstat)
{
	bool eogop = (1 == 0); //end of gop
	int frame_count = 0;

	drop_frame_flag = mpstat.get_drop_frame_flag();
	time_code_hours = mpstat.get_time_code_hours();
	time_code_minutes = mpstat.get_time_code_minutes();
	time_code_seconds = mpstat.get_time_code_seconds();
	time_code_pictures = mpstat.get_time_code_pictures();
	
//	write();
	while(!eogop)
	{
		//FIXME change when B and P pics are added
		pic = new PICTURE(1, frame_count, mpstat);
		pic->read_picture();
		pic->code_picture();
		pic->write_picture();
		delete pic;
		frame_count++;
		if(frame_count > 6)
			eogop = (1 == 1);
	} //end while
} //end GOP()

void GOP::write(void)
{
	outbuff.byte_allign();

	outbuff.put_bits(1,24);	//start code
	outbuff.put_bits(group_start_code, 8);
	outbuff.put_bits(drop_frame_flag, 1);
	outbuff.put_bits(time_code_hours, 5);
	outbuff.put_bits(time_code_minutes, 6);
	outbuff.put_bits(marker_bit, 1);
	outbuff.put_bits(time_code_seconds, 6);
	outbuff.put_bits(time_code_pictures, 6);
	outbuff.put_bits(closed_gop, 1);
	outbuff.put_bits(broken_link, 1);
} //end write()
/*****************************************************************************/
SEQUENCE::SEQUENCE(void)
{
	int gop_count = 0;
	bool eos = (1 == 0); //end of sequence

	horizontal_size_value = mpstat.get_horizontal_size_value();
	vertical_size_value = mpstat.get_vertical_size_value();
	aspect_ratio_information = mpstat.get_aspect_ratio_information();
	frame_rate_code = mpstat.get_frame_rate_code();
	load_intra_quantiser_matrix = mpstat.get_load_intra_quantiser_matrix();
	load_non_intra_quantiser_matrix = mpstat.get_load_non_intra_quantiser_matrix();


	//FIXME need to get these from mpstat or somewhere else
	bit_rate_value = 3750;
	vbv_buffer_size_value = 66;

	write();
	seq_ext.write();

	while(!eos)
	{
		gop = new GOP(mpstat);
		delete gop;
		gop_count++;
		if(gop_count > 60)
			eos = (1 == 1);
	}
}

void SEQUENCE::write()
{
	outbuff.byte_allign();

	outbuff.put_bits(1,24);	//start code
	outbuff.put_bits(sequence_header_code, 8);
	outbuff.put_bits(horizontal_size_value, 12);
	outbuff.put_bits(vertical_size_value, 12);
	outbuff.put_bits(aspect_ratio_information, 4);
	outbuff.put_bits(frame_rate_code ,4);
	outbuff.put_bits(bit_rate_value, 18);
	outbuff.put_bits(marker_bit, 1);
	outbuff.put_bits(vbv_buffer_size_value, 10);
	outbuff.put_bits(constrained_param_flag, 1);
	outbuff.put_bits(load_intra_quantiser_matrix, 1);
	outbuff.put_bits(load_non_intra_quantiser_matrix, 1);

}// end write()
SEQUENCE::~SEQUENCE()
{
//	delete gop;

	outbuff.byte_allign();
	outbuff.put_bits(1,24);
	outbuff.put_bits(0xB7,8); // write sequence_end_code
}
/*****************************************************************************/
















