/***************************************************************************
                          stats.cpp  -  description
                             -------------------
    begin                : Sat Dec 18 1999
    copyright            : (C) 1999 by Jeff Anderson
    email                : jbanders@mines.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef STATS_H
#define STATS_H
#include "stats.h"
#endif

#ifndef NULL
# define NULL 0
#endif

#include <string>

STATS::STATS()
{
	horizontal_size = 0; //352;
	vertical_size = 0; //240;
	frame_rate_code = 2; // 0010 in binary form

	aspect_ratio = 0x01;

	bit_rate = 3750;
	vbv_buffer_size = 66;

	intra_quantiser_matrix = NULL;
	non_intra_quantiser_matrix = NULL;

	temporal_reference = 0;
//	picture_coding_type = 1; //001 in binary form
	vbv_delay = 0xFFFF;
	
	alternate_scan = (1 == 0);
} // end STATS()

//return 12 least significant bits of horizontal_size
u16 STATS::get_horizontal_size_value()
{
	return horizontal_size;
} //end get_horizontal_size_value()

u32 STATS::get_horizontal_size(void)
{
	return horizontal_size;
} //end get_horizontal_size()

u32 STATS::get_color_horizontal_size(void)
{
	return (horizontal_size/2);
} //end get_color_horizontal_size(void)

u16 STATS::get_vertical_size_value(void)
{
	return vertical_size;
} //end get_vertical_size_value()

u32 STATS::get_vertical_size(void)
{
	return vertical_size;
} //end get_vertical_size()

u32 STATS::get_color_vertical_size(void)
{
	return (vertical_size/2);
} //end get_color_vertical_size()

BYTE_t STATS::get_aspect_ratio_information(void)
{
	return aspect_ratio;
} //end get_aspect_ratio_information()

BYTE_t STATS::get_frame_rate_code(void)
{
	return frame_rate_code;
} //end get_frame_rate_code()

BYTE_t STATS::get_load_intra_quantiser_matrix(void)
{
	if(intra_quantiser_matrix == NULL)
		return 0;
	else
		return 1;
} //end get_load_intra_quantiser_matrix()


BYTE_t STATS::get_load_non_intra_quantiser_matrix(void)
{
	if(non_intra_quantiser_matrix == NULL)
		return 0;
	else
		return 1;
} //end get_load_intra_quantiser_matrix()

BYTE_t STATS::get_drop_frame_flag(void)
{
	if(frame_rate_code == 4)
		return 1;
	else
		return 0;
} //end get_drop_frame_flag()

BYTE_t STATS::get_time_code_hours(void)
{
	return time_code_hours;
} // end get_time_code_hours()

BYTE_t STATS::get_time_code_minutes(void)
{
	return time_code_minutes;
} //end get_time_code_minutes()

BYTE_t STATS::get_time_code_seconds(void)
{
	return time_code_seconds;
} //end get_time_code_seconds()

BYTE_t STATS::get_time_code_pictures(void)
{
	return time_code_pictures;
}	//end get_time_code_pictures()

u16 STATS::get_temporal_reference(void)
{
	return temporal_reference;
}

u16 STATS::get_vbv_delay(void)
{
	return vbv_delay;
}
u16 STATS::get_numb_slice(void)
{
 	return 	((vertical_size + 15) / 16);
}

u32 STATS::get_slice_length(void)
{
 	return 	((horizontal_size + 15) / 16);
}

bool STATS::get_scan_type(void)
{
 	return alternate_scan;
}

void STATS::set_vertical_size(int vert_size)
{
	vertical_size = vert_size;
}

void STATS::set_horizontal_size(int horz_size)
{
	horizontal_size = horz_size;
}

void STATS::set_frame_rate(char *frame_rate)
{
	//FIXME add support for other frame rates
	if(strcmp(frame_rate,"24") == 0)
		frame_rate_code = 2;
	else if(strcmp(frame_rate,"25") == 0)
		frame_rate_code = 3;
	else if(strcmp(frame_rate,"30") == 0)
		frame_rate_code = 5;
	else if(strcmp(frame_rate,"50") == 0)
		frame_rate_code = 6;
	else if(strcmp(frame_rate,"60") == 0)
		frame_rate_code = 8;
	else
	{
		cout << "bad frame rate" << endl;
		exit(1);
	}
}
/*
	frame_rate_value

	frame_rate_code		frame_rate_value
	0000							Forbidden
	0001							24/1.001 (23.976...)
	0010							24
	0011							25
	0100							30.000/1.001 (29.97...)
	0101							30
	0110							50
	0111							60.000/1.001 (59.94...)
	1000							60
	1001							Reserved
	...								...
	1111							Reserved
*/
