
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#define SINGLE_BUFFER

#ifndef __BUF_H__
#define __BUF_H__

#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <pthread.h>

/***** for multiple buffers *****/

#define BUF_ANY		0
#define BUF_AUDIO	1
#define BUF_VIDEO	2
#define BUF_SUBPIC	3
#define BUF_OTHER	4
#define _BUF_MAX	5

#define BUF_DEPTH	200

#define BUF_FLAG_NONE		0x00
#define BUF_FLAG_CLOCK_VALID	0x01
#define BUF_FLAG_PTS_VALID	0x02

#define BUF_TYPE_AC3	0x80
#define BUF_TYPE_LPCM	0xA0
#define BUF_TYPE_MPEG1	0x01
#define BUF_TYPE_MPEG2	0x02
#define BUF_TYPE_MP3	0x03

typedef struct {
	u_char *mem;	// pointer to memory in buffer  (f.e. PS stream)
	u_char *data;	// pointer to data in buffer	(f.e. ES stream)
	int data_len;	// length of data in buffer
	int mem_len;	// length of memory in buffer
	u_int buf_id;	// data identifier

	u_char flags;
	long long clock;// clock
	long clock_ext; // clock extension
	u_int64_t pts;	// presentation time stamp

	u_int type;	// data type
} buf_entry_t;

typedef struct {
#ifndef SINGLE_BUFFER
	buf_entry_t entry[_BUF_MAX][BUF_DEPTH];
#else
	buf_entry_t entry[BUF_DEPTH];
#endif
	struct {
		u_int read;
		u_int write;

		pthread_mutex_t mutex;
		pthread_mutex_t mutex_slots_left;
		pthread_mutex_t mutex_new_data;
		pthread_cond_t cond_slots_left;
		pthread_cond_t cond_new_data;
		u_int wait4new;
		int forbid_write;
#ifndef SINGLE_BUFFER
	} ctrl[_BUF_MAX];
#else
	} ctrl;
#endif
	long long clock;
	struct timeval time;
} buf_t;

#if 0
int DecoderBufInit (buf_t *buf);
int DecoderBufExit (buf_t *buf);
void DecoderBufFlush (buf_t *buf, u_int buf_id);
int DecoderBufPut (buf_t *buf, buf_entry_t *buf_entry);
int DecoderBufGet (buf_t *buf, u_int buf_id, buf_entry_t *buf_entry);
void bufSetTime (buf_t *buf, u_int64_t clock);
int bufGetDTime (buf_t *buf, u_int64_t clock);
#else

int demuxBufInit (buf_t *buf);
int demuxBufExit (buf_t *buf);
void demuxBufFlush (buf_t *buf, u_int buf_id);
int demuxBufPut (buf_t *buf, buf_entry_t *buf_entry);
int demuxBufGet (buf_t *buf, u_int buf_id, buf_entry_t *entry);
inline void bufSetTime (buf_t *buf, u_int64_t clock);
inline int bufGetDTime (buf_t *buf, u_int64_t clock);
#endif

#endif
