/*
 *
 * Copyright (C) 2000 Yuqing Deng <Yuqing_Deng@brown.edu>
 *
 * navigation packet structure from vobdump by Eric Smith <eric@brouhaha.com>
 * 
 * program organization and bit stream buffering stolen from extract_ac3.c
 * by Aaron Holtzman <aholtzma@engr.uvir.ca>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *------------------------------------------------------------  
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <byteswap.h>


#define BUFSIZE 2048 /* needs to be as big as biggest header */

extern void ifoPrintVMOP(u_char *opcode);

typedef struct {
  u_int nv_pck_lbn : 32;  // logic block number
  u_short vobu_cat : 16;      // vobu category
  u_short reserved : 16;
  u_int vobu_uop_ctl : 32; // user operation
  u_int vobu_s_ptm : 32 ;
  u_int vobu_e_ptm : 32;
  u_int vobu_se_e_ptm : 32;
  u_int e_eltm : 32;       // ? elspe time
  u_char vobu_isrc[32];    // information source?
  // non seamless angle cell ??? address
  u_int nsml_agli_dsta[9];
} nav_pci_gi;


// highlight info

typedef struct {
  u_char reserved : 8;
  u_char hli_ss : 2; // 6 bit reserved, 2 bit hli_ss
  u_char reserved0 : 6;
  u_char   hli_s_ptm[4];
  u_char   hli_e_ptm[4];
  u_char   btn_se_e_ptm[4];

  u_char btngr0_dsp_ty : 3;
  u_char reserved2 : 1;
  u_char  btngr_ns : 2;
  u_char reserved1 : 2;

  u_char btngr2_dsp_ty : 3;
  u_char reserved4 : 1;
  u_char btngr1_dsp_ty : 3;
  u_char reserved3 : 1;

  u_char  btn_ofn : 8;
  u_char  btn_ns : 6;   // 2 reserved, 6 btn_ns
  u_char  reserved5 : 2;
  u_char  nsl_btn_ns : 6; // 2 reserved, 6 nsl_btn_ns
  u_char  reserved6 : 2;
  u_char  reserved7 : 8;

  u_char  fosl_btnn : 6; // 2 reserved, 6 nsl_btn_ns
  u_char  reserved8 : 2;

  u_char  foac_btnn : 6; // 2 reserved, 6 foac_btnn
  u_char  reserved9 : 2;

  u_char  btn_coli_0_sl[4];
  u_char  btn_coli_0_ac[4];
  u_char  btn_coli_1_sl[4];
  u_char  btn_coli_1_ac[4];
  u_char  btn_coli_2_sl[4];
  u_char  btn_coli_2_ac[4];
} nav_pci_hli_gi;


typedef struct {
  u_char x_start_f6 : 6; // first 6 bits of x_start
  u_char btn_coln : 2; // 2 button color number
  u_char x_end_f2 : 2; // first 2 bits of x_end
  u_char reserved : 2;
  u_char x_start_l4 : 4; // last 4 bits of x_start
  u_char x_end_l8 : 8; // last 8 bits of x_end
  
  u_char y_start_f6 : 6; // first 6 bits of y_start
  u_char auto_action_mode : 2; // 2
  u_char y_end_f2 : 2; // first 2 bits of y_end
  u_char reserved1 : 2;
  u_char y_start_l4 : 4; // last 4 bits of y_start
  u_char y_end_l8 : 8;

  u_char up : 6; // 6 bits up button number
  u_char reserved2 : 2;
  u_char down : 6; // down button number
  u_char reserved3 : 2;
  u_char left: 6;
  u_char reserved4 : 2;
  u_char right: 6;
  u_char reserved5 : 2;

  u_char btn_command[8];
} nav_pci_btnit;

typedef struct {
  u_int nv_pck_scr :32;
  u_int nv_pck_lbn : 32;  // logic block number
  u_int vobu_ea : 32;      // vobu category
  u_int vobu_1stref_ea : 32;
  u_int vobu_2ndref_ea : 32;
  u_int vobu_3rdref_ea : 32;
  u_short vobu_vob_idn : 16; // vob id
  u_char reserved : 8;
  u_char vobu_c_idn : 8;    // cell id
  u_int c_eltm : 32;       // cell elspe time
} nav_dsi_gi;

static char user_op[][40] = {
  "Title/Time Play",
  "Chapter Search/Play",
  "Title Play",
  "Stop",
  "GoUp",
  "Time/Chapter Search",
  "Prev/Top PG",
  "NextPG",
  "Forward Scan",
  "Backward Scan",
  "Title Menu Call",
  "Root Menu Call",
  "SubPic Menu Call",
  "Audio Menu Call",
  "Angle Menu Call",
  "Chapter Menu Call",
  "Resume",
  "Button Select/Activate",
  "Still Off",
  "Pause On",
  "Audio Stream Change",
  "SubPic Stream Change",
  "Angle Stream Change",
  "Karaoke Audio Pres Mode Change",
  "Video Pres Mode Change"
};

static int vobf;
static unsigned char buf[BUFSIZE];
static unsigned char *cur_pos;
static unsigned char *end_pos;
static u_int nbtns;
static u_int btn_grp;

void print_user_op(u_int u_op)
{
	int i;

	fprintf(stdout, "allowed user operations:\n");
	for (i = 0; i < 25; i++) {
		if (!(u_op & (0x1 << i))) {
			fprintf(stdout, "\t\t%s\n", user_op[i]);
		}
	}
}

void print_pci_gi(u_char *pci)
{
        int i;

        nav_pci_gi *gi = (nav_pci_gi*) pci;
	fprintf(stdout, "current block:   0x%08x\n", bswap_32(gi->nv_pck_lbn));
	fprintf(stdout, "vobu_cat:        0x%04x\n", bswap_16(gi->vobu_cat));

	if (gi->reserved)
		fprintf(stdout, "reserved:        0x%04x\n", gi->reserved);
	//fprintf(stdout, "user operations: 0x%08x\n", bswap_32(gi->vobu_uop_ctl));
	print_user_op(bswap_32(gi->vobu_uop_ctl));
	fprintf(stdout, "vobu_s_ptm:      0x%08x\n", bswap_32(gi->vobu_s_ptm));
	fprintf(stdout, "vobu_e_ptm:      0x%08x\n", bswap_32(gi->vobu_e_ptm));
	fprintf(stdout, "vobu_se_e_ptm:   0x%08x\n", bswap_32(gi->vobu_se_e_ptm));
	fprintf(stdout, "e_eltm:          0x%08x\n", bswap_32(gi->e_eltm));

	fprintf(stdout, "vobu_isrc:     \"");
	for (i = 0; i < 32; i++) {
	  u_char c;
	  if ((c = gi->vobu_isrc[i]) >= ' ' && (c <= '~'))
	    fprintf(stdout, "%c", c);
	  else
	    fprintf(stdout, ".");
	}
	fprintf(stdout, "\"\n");

	for (i = 0; i < 9; i++) {
	  fprintf(stdout, "nsml_agl_c%d_dsta:  0x%08x\n", i, bswap_32(gi->nsml_agli_dsta[i]));
	}
}

void print_pci_hli(u_char *pci)
{
        nav_pci_hli_gi *hli = (nav_pci_hli_gi*) pci;
	
	if (hli->reserved)
	  fprintf(stdout, "reserved not zero:        0x%04x\n", hli->reserved);
	if (hli->reserved0)
	  fprintf(stdout, "reserved0 not zero:       0x%02x\n", hli->reserved0);
	fprintf(stdout, "hli_ss:          0x%01x\n", hli->hli_ss);

	btn_grp = hli->btngr_ns;
	nbtns = hli->btn_ns;

	fprintf(stdout, "hli_s_ptm:       0x%02x%02x%02x%02x\n",
		hli->hli_s_ptm[0], hli->hli_s_ptm[1],
		hli->hli_s_ptm[2], hli->hli_s_ptm[3]);
	fprintf(stdout, "hli_e_ptm:       0x%02x%02x%02x%02x\n",
		hli->hli_e_ptm[0], hli->hli_e_ptm[1],
		hli->hli_e_ptm[2], hli->hli_e_ptm[3]);
	fprintf(stdout, "btn_se_e_ptm:    0x%02x%02x%02x%02x\n",
		hli->btn_se_e_ptm[0], hli->btn_se_e_ptm[1],
		hli->btn_se_e_ptm[2], hli->btn_se_e_ptm[3]);
	
	if (hli->reserved1)
	  fprintf(stdout, "reserved1 not zero:        0x%01x\n", hli->reserved1);
	fprintf(stdout, "button groups:   0x%01x\n", hli->btngr_ns);
	if (hli->reserved2)
	  fprintf(stdout, "reserved2 not zero:        0x%01x\n", hli->reserved2);
	fprintf(stdout, "btngr0_dsp_ty:   0x%01x\n", hli->btngr0_dsp_ty);
	if (hli->reserved3)
	  fprintf(stdout, "reserved3 not zero:        0x%01x\n", hli->reserved3);
	fprintf(stdout, "btngr1_dsp_ty:   0x%01x\n", hli->btngr1_dsp_ty);
	if (hli->reserved4)
	  fprintf(stdout, "reserved4 not zero:        0x%01x\n", hli->reserved4);
	fprintf(stdout, "btngr2_dsp_ty:   0x%01x\n", hli->btngr2_dsp_ty);
	
	fprintf(stdout, "btn_ofn:         0x%02x\n", hli->btn_ofn);
	if (hli->reserved5)
	fprintf(stdout, "reserved5 not zero:        0x%01x\n", hli->reserved5);
	fprintf(stdout, "btn_ns:          0x%02x\n", hli->btn_ns);

	if (hli->reserved6)
	  fprintf(stdout, "reserved6 not zero:        0x%01x\n", hli->reserved6);
	fprintf(stdout, "nsl_btn_ns:      0x%02x\n", hli->nsl_btn_ns);

	if (hli->reserved7)
	  fprintf(stdout, "reserved7 not zero:        0x%02x\n", hli->reserved7);

	if (hli->reserved8)
	  fprintf(stdout, "reserved8 not zero:        0x%01x\n", hli->reserved8);

	fprintf(stdout, "fosl_btnn:       0x%02x\n", hli->fosl_btnn);

	if (hli->reserved9)
	  fprintf(stdout, "reserved9 not zero:        0x%01x\n", hli->reserved9);
	fprintf(stdout, "foac_btnn:       0x%02x\n", hli->foac_btnn);
	fprintf(stdout, "btn_coli_0 sl: 0x%02x%02x%02x%02x ac: 0x%02x%02x%02x%02x\n",
		hli->btn_coli_0_sl[0], hli->btn_coli_0_sl[1],
		hli->btn_coli_0_sl[2], hli->btn_coli_0_sl[3],
		hli->btn_coli_0_ac[0], hli->btn_coli_0_ac[1],
		hli->btn_coli_0_ac[2], hli->btn_coli_0_ac[3]);
	fprintf(stdout, "btn_coli_1 sl: 0x%02x%02x%02x%02x ac: 0x%02x%02x%02x%02x\n",
		hli->btn_coli_1_sl[0], hli->btn_coli_1_sl[1],
		hli->btn_coli_1_sl[2], hli->btn_coli_1_sl[3],
		hli->btn_coli_1_ac[0], hli->btn_coli_1_ac[1],
		hli->btn_coli_1_ac[2], hli->btn_coli_1_ac[3]);
	fprintf(stdout, "btn_coli_2 sl: 0x%02x%02x%02x%02x ac: 0x%02x%02x%02x%02x\n",
		hli->btn_coli_2_sl[0], hli->btn_coli_2_sl[1],
		hli->btn_coli_2_sl[2], hli->btn_coli_2_sl[3],
		hli->btn_coli_2_ac[0], hli->btn_coli_2_ac[1],
		hli->btn_coli_2_ac[2], hli->btn_coli_2_ac[3]);
       
}
	
void print_pci_btns(u_char* pci, u_int nbtns, u_int btn_gprs)
{
  
  int i, j;

  for (i = 0; i < btn_gprs; i++) {
    for (j = 0; j < 36 / btn_gprs; j++, pci += 18) {
      if (j < nbtns) {
	nav_pci_btnit *btnit = (nav_pci_btnit *) pci;
	u_int x_start, y_start, x_end, y_end;
	x_start = btnit->x_start_f6;
	x_start <<= 4; x_start += btnit->x_start_l4;
	y_start = btnit->y_start_f6;
	y_start <<= 4; y_start += btnit->y_start_l4;
	x_end = btnit->x_end_f2;
	x_end <<= 8; x_end += btnit->x_end_l8;
	y_end = btnit->y_end_f2;
	y_end <<= 8; y_end += btnit->y_end_l8;
	
	fprintf (stdout, "group %d btni %d:  ", i, j);
	fprintf (stdout, "btn_coln %d, auto_action_mode %d\n",
		 btnit->btn_coln, btnit->auto_action_mode); 
	fprintf (stdout, "coords   (0x%03x, 0x%03x) .. (0x%03x, 0x%03x)\n",
		 x_start, y_start, x_end, y_end);                                
	fprintf (stdout, "up %d, ", btnit->up);  
	fprintf (stdout, "down %d, ", btnit->down); 
	fprintf (stdout, "left %d, ", btnit->left);
	fprintf (stdout, "right %d\n", btnit->right);
	
	ifoPrintVMOP(&(btnit->btn_command[0]));
	fprintf (stdout, "\n");	     
      }
    }
  }
}
                                                               
void print_pci(u_char* pci)
{
        u_char *cur = pci;
	if (*cur++ == 0x00) {
	  fprintf(stdout, "pci packet\n");
	} else {
	  fprintf(stdout, "not a pci packet\n");
	}
	print_pci_gi(cur);
	cur += sizeof(nav_pci_gi);
	print_pci_hli(cur);
	cur += 46;
	print_pci_btns(cur, nbtns, btn_grp);
}

void print_dsi_gi(u_char *gi)
{
        nav_dsi_gi *dsi_gi = (nav_dsi_gi *) gi;
	
	  fprintf (stdout, "nv_pck_scr     0x%08x\n", bswap_32(dsi_gi->nv_pck_scr));
	  fprintf (stdout, "nv_pck_lbn     0x%08x\n", bswap_32(dsi_gi->nv_pck_lbn));
	  fprintf (stdout, "vobu_ea        0x%08x\n", bswap_32(dsi_gi->vobu_ea));
	  fprintf (stdout, "vobu_1stref_ea 0x%08x\n", bswap_32(dsi_gi->vobu_1stref_ea));
	  fprintf (stdout, "vobu_2ndref_ea 0x%08x\n", bswap_32(dsi_gi->vobu_2ndref_ea));
	  fprintf (stdout, "vobu_3rdref_ea 0x%08x\n", bswap_32(dsi_gi->vobu_3rdref_ea));
	  fprintf (stdout, "vob id:        0x%04x\n", bswap_16(dsi_gi->vobu_vob_idn));
	  fprintf (stdout, "reserved:      0x%02x\n", dsi_gi->reserved);
	  fprintf (stdout, "cell id:       0x%02x\n", dsi_gi->vobu_c_idn);
	  fprintf (stdout, "c_eltm:        0x%04x\n", bswap_32(dsi_gi->c_eltm));
}

void print_dsi(u_char* dsi)
{
        u_char *cur = dsi;
	if (*cur++ == 0x01) {
	  fprintf(stdout, "dsi pack\n");
	} else {
	  fprintf(stdout, "not a dsi packet\n");
	}
	print_dsi_gi(cur);
	cur += sizeof(nav_dsi_gi);
	//print_dsi_sml_pbi(cur);
	// cur += 148;
	// print_dsi_sml_anli(cur);
	// cur += 54;
	// print_vobu_sri(cur);
	// cur += 168;
	// print_synci(cur); // 144 bytes
}

void file_init(char file_name[])
{
	if(file_name[0] == '-' && file_name[1] == '\0')
	{
	   vobf = STDIN_FILENO;
	}
	else if ((vobf = open(file_name, O_RDONLY)) < 0)
	{
		fprintf(stdout,"File not found\n");
		exit(1);
	}
	cur_pos = buf;
	end_pos = buf;
}

inline void increment_position(long x)
{
	if(cur_pos + x <= end_pos)
	{
	   cur_pos += x;
	   if(cur_pos == end_pos)
	   {
		cur_pos = buf;
                end_pos = buf;
	   }
	}
	else
	{
           long size = 0;
           x -= (long)(end_pos - cur_pos);
#ifdef SEEK_PIPES
		if(lseek(vobf, x, SEEK_CUR) < 0)
		{
		   fprintf(stdout, "Error: unexpected end of stream\n");
		   exit(1);
		}
#else
		while(x)
		{
		   size = (x > BUFSIZE) ? BUFSIZE : x;
		   if(read(vobf, buf, size) < size)
		   {
		      fprintf(stdout, "Error: unexpected end of stream\n");
		      exit(1);
		   }
                   x-=size;
		}
#endif
		cur_pos = buf;
                end_pos = buf;
	}
}

inline static void load_next_bytes(long count)
{
   if(cur_pos + count <= end_pos)
      return;
   if(cur_pos + count > buf + BUFSIZE - 1 )
   {
      printf ("No buffer space to read %ld bytes\n", count);
      exit(1);
   }

   count -= (long)(end_pos - cur_pos);
   if(read(vobf, end_pos, count) < count)
   {
      fprintf(stdout, "Error: unexpected end of stream\n");
      exit(1);
   }
   end_pos += count;
}

inline int next_24_bits(long x)
{
   load_next_bytes(3);
	if (cur_pos[0] != ((x >> 16) & 0xff))
		return 0;
	if (cur_pos[1] != ((x >>  8) & 0xff))
		return 0;
	if (cur_pos[2] != ((x      ) & 0xff))
		return 0;

	return 1;
}

inline int next_32_bits(long x)
{
   load_next_bytes(4);
	if (cur_pos[0] != ((x >> 24) & 0xff))
		return 0;
	if (cur_pos[1] != ((x >> 16) & 0xff))
		return 0;
	if (cur_pos[2] != ((x >>  8) & 0xff))
		return 0;
	if (cur_pos[3] != ((x      ) & 0xff))
		return 0;

	return 1;
}

void read_write_next_bytes(long count, int outfd)
{
   long size;
   size = (long)(end_pos - cur_pos);
   if(size > count)
   {
      write(outfd, cur_pos, count);
      cur_pos +=count;
      if(cur_pos == end_pos)
      {
	 cur_pos = buf;
	 end_pos = buf;
      }
      return;
   }
   else if(size > 0)
   {
      write(outfd, cur_pos, size);
   }

   while(count)
   {
      size = (count > BUFSIZE) ? BUFSIZE : count;
      if(read(vobf, buf, size) < size ||
	 write(outfd, buf, size) < size)
      {
	 fprintf(stdout, "Error: unexpected end of stream\n");
      }
      count -= size;
   }
   cur_pos = buf;
   end_pos = buf;
}

void parse_pes(void)
{
	unsigned long data_length;
	unsigned long header_length;

	load_next_bytes(9);

	//The header length is the PES_header_data_length byte plus 6 for the packet
	//start code and packet size, 3 for the PES_header_data_length and two
	//misc bytes, and finally 4 bytes for the mystery AC3 packet tag 
	header_length = cur_pos[8] + 6 + 3;
	data_length =(cur_pos[4]<<8) + cur_pos[5];

	if(cur_pos[3] == 0xbf)
	{
		load_next_bytes(data_length - 3);

		// pci packet
		fprintf(stdout,"start of pci curpos[] = %02x%02x%02x%02x\n",
			cur_pos[0],cur_pos[1],cur_pos[2],cur_pos[3]);
		fprintf(stdout,"data_length = %x\n", data_length);
		print_pci(&cur_pos[6]);
		
		increment_position(data_length + 6);

		// dsi packet
		load_next_bytes(9);

		data_length = (cur_pos[4] << 8) + cur_pos[5];
		fprintf(stdout,"start of dsi curpos[] = %02x%02x%02x%02x\n",
			cur_pos[0],cur_pos[1],cur_pos[2],cur_pos[3]);
		fprintf(stdout,"data_length = %x\n", data_length);
		load_next_bytes(data_length - 3);
		print_dsi(&cur_pos[6]);
		
		increment_position(data_length + 6);
	} else {
	  increment_position(data_length + 6);
	}
}

void parse_pack(void)
{
	unsigned long skip_length;

	// Deal with the pack header 
	// The first 13 bytes are junk. The fourteenth byte 
	// contains the number of stuff bytes 
	load_next_bytes(14);
	skip_length = cur_pos[13] & 0x7;
	increment_position(14 + skip_length);

	// Deal with the system header if it exists 
	if(next_32_bits(0x000001bb))
	{
	// Bytes 5 and 6 contain the length of the header minus 6
                load_next_bytes(6);
		skip_length = (cur_pos[4] << 8) +  cur_pos[5];
		increment_position(6 + skip_length);
	}

	while(next_24_bits(0x000001) && !next_32_bits(0x000001ba))
	{
		parse_pes();
	}
}

int main(int argc, char *argv[])
{

	if (argc < 2) {
		fprintf(stdout, "usage: %s mpeg_stream [track number]\n", argv[0]);
		exit(1);
	}

	file_init(argv[1]);

	if(!next_32_bits(0x000001ba))
	{
		fprintf(stdout, "Non-program streams not handled - exiting\n\n");
		exit(1);
	}

	do
	{
		parse_pack();
	} 
	while(next_32_bits(0x000001ba));

	fprintf(stdout,"curpos[] = %x%x%x%x\n",cur_pos[0],cur_pos[1],cur_pos[2],cur_pos[3]);

	if(!next_32_bits(0x000001b9))
	{
		fprintf(stdout, "Error: expected end of stream code\n");
		exit(1);
	}

	if(vobf != STDIN_FILENO) close(vobf);
	return 0;
}		

