
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>

#include <oms/buf.h>


/**
 *
 */

int demuxBufInit (buf_t *buf)
{
	int i;

	pthread_mutex_init(&(buf->ctrl.mutex_slots_left), NULL);
	pthread_cond_init (&(buf->ctrl.cond_slots_left), NULL);
	pthread_mutex_init(&(buf->ctrl.mutex_new_data), NULL);
	pthread_cond_init (&(buf->ctrl.cond_new_data), NULL);
	buf->ctrl.forbid_write = 0;

	for (i=0; i<BUF_DEPTH; i++) {
		buf->entry[i].mem_len = 0;
		buf->entry[i].data_len = 0;
		buf->entry[i].mem = NULL;
		buf->entry[i].data = NULL;
	}

	return 0;
}


/**
 *
 */

int demuxBufExit (buf_t *buf)
{
	demuxBufFlush (buf, BUF_ANY);

	return 0;
}


/**
 * Ever used a toilet? - then you should be familiar with the concept
 * behind this function
 */

void demuxBufFlush (buf_t *buf, u_int buf_id)
{
	buf_entry_t entry;

	while (demuxBufGet (buf, buf_id, &entry))
		free (entry.data);
}


// simple implementation of circular queues
/**
 *
 */

int demuxBufPut (buf_t *buf, buf_entry_t *buf_entry)
{
	u_int write_index;

//        pthread_cond_signal (&(buf->ctrl.cond_new_data));

	memcpy (&(buf->entry[buf->ctrl.write]), buf_entry, sizeof (buf_entry_t));

	write_index = (buf->ctrl.write + 1) % BUF_DEPTH;

	if (write_index == buf->ctrl.read) {    // buffer full
		buf->ctrl.forbid_write = 1;

		pthread_cond_wait (&(buf->ctrl.cond_slots_left), &(buf->ctrl.mutex_slots_left));
	}

	buf->ctrl.write = write_index;

	return 0;
}


/**
 *
 */

int demuxBufGet (buf_t *buf, u_int buf_id, buf_entry_t *entry)
{
	u_int buf_fill;

	if (buf->ctrl.read == buf->ctrl.write)		// no new entry
		return -1;

	memcpy (entry, &(buf->entry[buf->ctrl.read]), sizeof(buf_entry_t));

	buf->ctrl.read = (buf->ctrl.read + 1) % BUF_DEPTH;
pthread_cond_signal (&(buf->ctrl.cond_slots_left));

	if (buf->ctrl.forbid_write) {
		if (buf->ctrl.read <= buf->ctrl.write)
			buf_fill = buf->ctrl.write - buf->ctrl.read;
		else
			buf_fill = (BUF_DEPTH - buf->ctrl.read) + buf->ctrl.write;

// wait a little bit, until we allow reading again ...
// (high water = 100%, low water = 50%)

	}

	if (buf_fill <= BUF_DEPTH/2)
		pthread_cond_signal (&(buf->ctrl.cond_slots_left));

	return 0;
}


/**
 *
 */

inline void bufSetTime (buf_t *buf, u_int64_t clock)
{
	gettimeofday (&buf->time, 0L);
	buf->clock = clock;
}


/**
 *
 */

inline int bufGetDTime (buf_t *buf, u_int64_t clock)
{
	u_int64_t dclock = 0;             // where we should be
	u_int64_t dtime = 0;              // where we are
	struct timeval time;

	if (!buf->time.tv_sec &&  !buf->time.tv_sec)
		return 0;

	gettimeofday (&time, 0L);

	dclock = clock - buf->clock;    // where we should be
	dtime = (time.tv_sec - buf->time.tv_sec);
	dtime *= 1000000;
	dtime += (time.tv_usec - buf->time.tv_usec);

	if (dclock < 0)
		return -1;

	return dclock - dtime;
}

