
/*
 *
 * Copyright (C) 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */

//define NEW_PLUGIN

#include <stdio.h>
#include <dlfcn.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>

#include <config.h>
#include <oms/oms.h>
#include <oms/plugin.h>

static plugin_t *plugin_list[MAX_PLUGIN_ID];

/**
 * initialize plugins
 **/

int pluginInit (void)
{
	pluginScan (OMS_PLUGINDIR);

	return 0;
}


/**
 *
 **/

int pluginRegister (char *whoami, u_int plugin_id, u_int stream_id, void *plugin_data)
{
/*
        plugin_t *entry = calloc (1, sizeof (plugin_t));

        entry->whoami = strdup (whoami);
        entry->plugin_id = plugin_id;
        entry->stream_id = stream_id;
        entry->ops = plugin_data;
        entry->next = plugin_list [PLUGIN_MAJOR(plugin_id)];
*/
//#ifdef DEBUG
	fprintf (stderr, "registering plugin %s", whoami);
	fprintf (stderr, "\tplugin_id: %x\n", plugin_id);
	fprintf (stderr, "\tstream_id: %x\n", stream_id);
//#endif DEBUG

/*
        plugin_list [PLUGIN_MAJOR(plugin_id)] = entry;
*/
	return 0;
}


#ifdef NEW_PLUGIN
/**
 *
 **/

plugin_t *_findPlugin (u_int plugin_id, u_int stream_id)
{
        plugin_t *entry = plugin_list [PLUGIN_MAJOR (plugin_id)];

	while (entry) {
		if (entry->stream_id == stream_id)
			return entry;

		entry = entry->next;
	}

	return NULL; 
}


/**
 * Open and initialize plugin
 */

int pluginOpen (plugin_t *plugin[], u_int plugin_id, u_int stream_id, void *data)
{
        plugin_t *entry;

	if (PLUGIN_MAJOR(plugin_id) >= MAX_PLUGIN_ID) {
		fprintf (stderr, "pluginOpen: unknown plugin type\n");
		return -1;
	}

	if (!(entry = _findPlugin (plugin_id, stream_id))) {
		fprintf (stderr, "pluginOpen: cannot find plugin %x %x\n", plugin_id, stream_id);
		return -1;
	}

	plugin[PLUGIN_MAJOR(plugin_id)] = entry;

	return 0;
}

/**
 * Open plugin
 **/

void *_pluginOpen (plugin_t *plugin[], u_int plugin_id, u_int stream_id)
{
fprintf (stderr, "_pluginOpen\n");

/*
	omsPrint (OMS_PRINT_DEBUG, "try to open plugin %s", plugin_name);

	if (PLUGIN_MAJOR(plugin_id) >= MAX_PLUGIN_ID) {
		fprintf (stderr, "_pluginOpen: unknown plugin type\n");
		return NULL;
	}

	if (!(plugin->handle[plugin_id] = dlopen (plugin_name, RTLD_LAZY))) {
		fprintf (stderr, "PLUGIN: %s:%s doesn't seem to be installed\n", plugin_name, dlerror ());

		return NULL;
	}

	omsPrint (OMS_PRINT_DEBUG, "plugin %s opened", plugin_name);

//	return plugin->handle[plugin_id];
*/
	return NULL;
}


/**
 * Close plugin and clean up plugin structure 
 **/

void *pluginClose (plugin_t *plugin, u_int plugin_id)
{
	if (PLUGIN_MAJOR(plugin_id) >= MAX_PLUGIN_ID) {
		fprintf (stderr, "pluginClose: unknown plugin type %x\n", plugin);
		return NULL;
	}

//DENT call plugin->*->close() here ...
//	if (plugin->handle[plugin_id])
//		dlclose (plugin->handle[plugin_id]);
//	plugin->ops[plugin_id] = NULL;

	return NULL;
}


/**
 *
 **/

// DENT: should work as follows (in the future)
//	1) compare fileextension
//		if extension matches, check if magic also matches
//	2) if no filename matches, check all extensions
//
//	OR ... just use it like now - just check magic codes
//	i think i'll implment it like now, but save all magic codes into a list
//	so i don't have to dlopen the plugin first.
 
static int _pluginGetDecapsMagic (plugin_t *plugin, char *plugin_name, u_char *buf)
{
/*
	plugin_decaps_magic_t *magic;
	int i;

	pluginOpen (plugin, PLUGIN_ID_DECAPS, plugin_name, NULL);
	magic = PLUGIN_OPS_DECAPS(plugin)->magic ();

	if (magic) {
		fprintf (stderr, "%s: file: %s\n", plugin_name, magic->file_ext);
		fprintf (stderr, "%s: len: %d\n", plugin_name, magic->magic_len);

		fprintf (stderr, "%s: magic: ", plugin_name);
		for (i=0; i<magic->magic_len; i++)
			fprintf (stderr, "0x%02x ", magic->magic[i]);
		fprintf (stderr, "\n\n");

		if (!(strncmp (buf, magic->magic, magic->magic_len))) {
			fprintf (stderr, "%s matches -> found the right decapsulation for our file\n", plugin_name);
			return 0;
		}

	} else
		fprintf (stderr, "%s: NO MAGIC!\n\n", plugin_name);

	pluginClose (plugin, PLUGIN_ID_DECAPS);
*/
	return -1;
}


/**
 * DENT: i know this is an ugly contruct right now, but:
 *	1) if we've an install routine, plugins will be looked up
 *		in the directory
 **/

int pluginGetDecapsMagic (plugin_t *plugin, char *filename)
{
/*
	u_char buf[20];
	int i;
	PLUGIN_OPS_INPUT(plugin)->seek (0);
	PLUGIN_OPS_INPUT(plugin)->read (buf, 20);

#ifdef DEBUG
	for (i=0; i<20; i++)
		fprintf (stderr, "0x%02x ", buf[i]);
#endif

#if 0
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_PS, buf))
		return 0;
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_TS, buf))
		return 0;
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_AVP, buf))
		return 0;
	if (!_pluginGetDecapsMagic (plugin, DL_DECAPS_VCD, buf))
		return 0;

// fallback
	pluginOpen (plugin, PLUGIN_ID_DECAPS, DL_DECAPS_DUMMY, NULL);
		return 0;
#endif
*/
	return -1;
} 

#endif NEW_PLUGIN

/**
 * filtering useful files
 **/

int scandir_select (const struct dirent *dir_entry)
{
	char *ext;

	if (!(ext = strrchr(dir_entry->d_name, '.')))
		return 0;

	if (strncasecmp (ext, ".so", 3))
		return 0;

	return 1;
}


/**
 * scanning for available plugins
 **/

int pluginScan (char *dir)
{
	static struct dirent **namelist;
	int i;

	if (!dir)
		return -1;

	printf ("Scanning \"%s\" for plugins\n", dir);

	if ((i = scandir (dir, &namelist, scandir_select, 0)) < 0)
		perror("scandir");
	else
		while (i--) {
			void *(*init)(void *);
			void *handle;
			char plugin_name[200];

			snprintf (plugin_name, 200, "%s/%s",  dir,  namelist[i]->d_name);
			fprintf (stderr, "checking for plugin >%s<\n", plugin_name);

			if (!(handle = dlopen (plugin_name, RTLD_LAZY))) {
				fprintf (stderr, "PLUGIN: %s doesn't seem to be installed (%s)\n", plugin_name, dlerror ());

				return -1;
			}


			if (!(init = dlsym (handle, "plugin_init"))) {
				fprintf (stderr, "PLUGIN: no plugin_init - failed to load %s (%s)\n", namelist[i]->d_name, dlerror ());
				continue;
			}
				

			if ((*init) (plugin_name)) {
				fprintf (stderr, "PLUGIN: failed to load %s (%s)\n", namelist[i]->d_name, dlerror ());
			}
		}

	return 0;
}
