
/*
 * AUDIO VIDEO PES DECAPSULATION 
 *
 * Copyright (C) 1999, 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>

#include <dvb/ps.h>
#include <dvb/pes.h>
#include <dvb/avp.h>
#include <oms/plugin/decaps.h>

static int _avp_open (struct plugin_decaps_struct *plugin);
static int _avp_close (struct plugin_decaps_struct *plugin);
static int _avp_decaps			(buf_t *buf);
static plugin_decaps_magic_t *_avp_magic (void);
static int _avp_set_filter		(u_int filter_id, int val);

static struct avp_priv_struct {
	u_int filter[_BUF_MAX];
} priv;

static plugin_decaps_t decaps = {
	_avp_open,
	_avp_close,
	_avp_decaps,	// decasulation
	_avp_magic,	// autodetect decapsulation
	_avp_set_filter,// filtering
	NULL,		// get_block (inserted)
	NULL,		// put_block (inserted)
	2048,		// blocksize (inserted: this is a default)
	NULL		// private data
};

static buf_entry_t buf_entry; 
static buf_t *buf; 

#define AUDIO_LPCM	0xA0
#define AUDIO_AC3	0x80
#define SPU_SUBTITLE	0x20

#define PS_FILTER_VIDEO 0xE0
#define PS_FILTER_SPU   0x20

static char avp_file_magic[] = {
       	'A', 'V' 
};

static plugin_decaps_magic_t avp_magic_entry = {
        "avp",
        2,
        avp_file_magic
};


/**
 *
 **/

static int _avp_open (struct plugin_decaps_struct *plugin)
{
	buf_entry.mem_len = decaps.blocksize;

	fprintf (stderr, "decaps_init called\n");

//DENT just for debugging purposes
	priv.filter[BUF_VIDEO] = 0xE0;
	priv.filter[BUF_SUBPIC] = -1;	// subtitle substream id (0..31,-1=off)
	priv.filter[BUF_AUDIO] = 0x00;	// audio substream id (0...7, -1=off)

	return 0;
}


/**
 *
 **/

static int _avp_close (struct plugin_decaps_struct *plugin)
{
	return 0;
}


#if 0
/**
 *
 */

static inline char *_get_bytes_block (u_int len)
{
	if (!buf_entry.mem)
		buf_entry.mem = malloc (decaps.blocksize);// check return val
	buf_entry.mem_size = decaps.blocksize;

	if (decaps.read (buf, &buf_entry) <= 0)
		return NULL;

	buf_entry.data = buf_entry.mem + len;

	return buf_entry.mem;
}
#else
#define _get_bytes_block _get_bytes
#endif


/**
 *
 */

static inline char *_get_bytes (u_int len)
{
#if 0
	if (buf_entry.mem) {
		free (buf_entry.mem)
		buf_entry.mem = malloc (len);// check return val
	}

	if (decaps.read (buf_entry.mem, len) <= 0)
		return NULL;

	return buf_entry.mem;
#else
	char *ptr = buf_entry.data;

	buf_entry.data += len;

	return ptr;
#endif
}


/**
 * Look if there's some useful data in this whole crap format
 */

static int _avp_decaps (buf_t *_buf)
{
	avp_hdr_t *avp;

	buf = _buf;

	if (!(avp = (avp_hdr_t *) _get_bytes_block (AVP_HDR_LEN)))
		return -1;

	buf_entry.flags = avp->pts_present ? BUF_FLAG_CLOCK_VALID : BUF_FLAG_NONE;

	switch (avp->stream_id) {
	case AVP_VIDEO:
		buf_entry.buf_id = BUF_VIDEO;
		fprintf (stderr, "AVP: video\n");
		break;
	case AVP_AUDIO:
		buf_entry.buf_id = BUF_AUDIO;
		fprintf (stderr, "AVP: audio\n");
		break;
	default:
		fprintf (stderr, "AVP: unhandled AVP id 0x%02x\n", avp->stream_id);
	}

	fprintf (stderr, "AVP: len: %d", avp->length_hi<<8 | avp->length_lo);

	//_get_bytes (avp->length_hi<<8 | avp->length_lo);
	fprintf (stderr, "AVP: fix me !!!\n");

// parse prefix and rest of hdr here
	return 0;
}


/**
 * Autodetect decapsulation method
 */

static plugin_decaps_magic_t *_avp_magic (void)
{
	return &avp_magic_entry;
}


/**
 * Not really a big deal in this format, but we can disable and
 * enabling stream, saving memoy bandwidth, and buffer space
 */

static int _avp_set_filter (u_int filter_id, int val)
{
	if (filter_id > _BUF_MAX)
		return -1;

	priv.filter[filter_id] = val;

	fprintf (stderr, "avp_set_filter called\n");
	return 0;
}


/**
 * Setup the plugin
 **/

void *plugin_init (char *whoami)
{
        pluginRegister (whoami,
                PLUGIN_ID_DECAPS,
                STREAM_ID_MPEG2,
                &decaps);

//FIXME
	_avp_open (&decaps);

	return &decaps;
}


/**
 * Have we created some mess here? - don't think so, but clean up anyway
 */

void plugin_exit (void)
{
	fprintf (stderr, "decaps_close called\n");
}

