#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/cdrom.h>
#include <string.h>

#include <oms/plugin/decaps.h>

static int _vcd_open (plugin_decaps_t *plugin);
static int _vcd_close (plugin_decaps_t *plugin);
static int vcd_decaps (buf_t *buf);
static plugin_decaps_magic_t *vcd_magic (void);
static int vcd_set_filter (u_int filter_id, int val);

typedef struct {
	char sync	[12],
	header		[4],
	subheader	[8],
	data		[2324],
	spare		[4];
} cdsector;

static plugin_decaps_t decaps = {
	_vcd_open,
	_vcd_close,
	vcd_decaps,
	vcd_magic,
	vcd_set_filter,
	NULL,
	NULL,
	sizeof(cdsector),
	NULL
};

static buf_entry_t buf_entry;


static int _vcd_open (plugin_decaps_t *plugin)
{
	return 0;
}


/**
 *
 **/

static int _vcd_close (plugin_decaps_t *plugin)
{
	return 0;
}


/**
 * The Decapsulation itself
 */

static int vcd_decaps (buf_t *buf)
{
	cdsector *buffer;

	if (!buf_entry.mem) {
		buf_entry.mem = malloc (sizeof(cdsector));
		buf_entry.mem_len = sizeof(cdsector);
	}

	buffer=(cdsector*)buf_entry.mem;

	if (decaps.read (buf, &buf_entry) <= 0)
		return -1;
  
	if (buffer->subheader[1]==1) {
		if ((buffer->subheader[2]==0x62) &&
			(buffer->subheader[3]==0x0f))
			buf_entry.buf_id = BUF_VIDEO;
		else if ((buffer->subheader[2]==0x64) &&
			(buffer->subheader[3]==0x7f))
			buf_entry.buf_id = BUF_AUDIO;
		else {
			fprintf (stderr, "Unhandled stream: %x%x\n",
				buffer->subheader[2],
				buffer->subheader[3]);
			return 0;
		}

		buf_entry.data = buffer->data;
		buf_entry.data_len = sizeof (buffer->data);
		buf_entry.mem = (u_char *) buffer;
		buf_entry.mem_len = decaps.blocksize;

		decaps.write (buf, &buf_entry);
		buf_entry.mem = NULL;
	}

	return 0;
}


/**
 * Autodetect decapsulation method, but this is just suitable for VCDs,
 *	so it should not be included into autodetection
 */
  
static plugin_decaps_magic_t *vcd_magic (void)
{
	return NULL;
}


/**
 *
 */

static int vcd_set_filter (u_int filter_id, int val)
{
	return 0;
}


/**
 *
 **/

void *plugin_init (char *whoami)
{
        pluginRegister (whoami,
                PLUGIN_ID_DECAPS,
                STREAM_ID_MPEG2,
                &decaps);

	return &decaps;
}


/**
 *
 **/

void plugin_close (void)
{
	fprintf (stderr, "decaps_close called\n");
}
