
/*
 *
 * Copyright (C) 1999, 2000  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


//#define TOAST_SPU
//#define TOAST_SYNC 1

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>

#include <oms/oms.h>
#include <oms/plugin.h>

#ifdef TOAST_SPU
#include "../../codec/spudec/spu.h"
#include "../../codec/spudec/overlay.h"
#include "../../codec/spudec/display.h"

static u_int *bg_image;
#endif
#include "../../codec/mpg123/mpg123.h"
#include "../../codec/mpg123/mpglib.h"


static int _sw_init (plugin_codec_t *plugin, buf_t *buf, char *output_path);
static int _sw_open (plugin_codec_t *plugin);
static int _sw_read (plugin_codec_t *plugin, buf_t *buf, buf_entry_t *buf_entry);
static int _sw_set_status (plugin_codec_t *plugin, u_int status);
static int _sw_set_attributes (plugin_codec_t *plugin, u_int id, void *attr);


static struct sw_priv_struct {
	int fd_video;
	int fd_audio;
	int fd_other;
#ifndef TOAST_SPU
	int fd_spu;
#endif
	buf_t *buf;
} sw_priv;


static plugin_codec_t codec_sw = {
	&sw_priv,
	_sw_init,	// init
	NULL,		// exit
	_sw_open,	// open
	NULL,		// close
	_sw_read,	// read
	NULL,		// write
	_sw_set_status,
	_sw_set_attributes,
}; 


/**
 *
 **/

static int _sw_open (plugin_codec_t *plugin)
{
	return 0;
}


/**
 *
 **/

static int _sw_read (plugin_codec_t *plugin, buf_t *buf, buf_entry_t *buf_entry)
{
	struct sw_priv_struct *priv = plugin->priv; 
//	static struct mpstr mp;	// for mp3

 //       InitMP3(&mp);

/*
	if (buf_entry->flags & BUF_FLAG_CLOCK_VALID) {
		u_int64_t clock = (buf_entry->clock * 300 + buf_entry->clock_ext)/27; // MHz
		bufSetTime (buf, clock);
	}
*/
	switch (buf_entry->buf_id) {
	case BUF_AUDIO: {
//fprintf (stderr, "A");
		if (priv->fd_audio < 0)
			break;

		switch (buf_entry->type) {
#if 0
			case BUF_TYPE_MP3: {	
				int out_len, ret;
				char out[8192];

				ret = decodeMP3 (&mp, buf_entry->data, buf_entry->data_len, out, 8192, &out_len);
					write (priv->fd_audio, out, out_len);
/*
				while (ret == MP3_OK) {
					write (priv->fd_audio, out, out_len);
					ret = decodeMP3 (&mp, buf_entry->data, buf_entry->data_len, out, 8192, &out_len);
				}
*/

				break;
			}
#endif
			default:
				write (priv->fd_audio, buf_entry->data, buf_entry->data_len);
			}
			break;
	}
	case BUF_OTHER:
		if (priv->fd_other >= 0) {
			fprintf (stderr, "other %d",  buf_entry->data_len);
			write (priv->fd_other, buf_entry->data, buf_entry->data_len);
		}

		break;
	case BUF_SUBPIC: {
#ifndef TOAST_SPU
		if (priv->fd_spu >= 0)
			write (priv->fd_spu, buf_entry->data, buf_entry->data_len);
#else
		struct spu_img_struct spu;

		if (!spuParseHdr (&spu, buf_entry->data-3, buf_entry->data_len+3)) {
			spuParseData (&spu);
			memset (bg_image, 0, 720 * 576 * sizeof(u_int));
			if ((bg_image = overlay (bg_image, &spu, 0.8)))
				display_frame (bg_image);
			free (spu.data);
		}
#endif
		break;
	}
	case BUF_VIDEO: {
//fprintf (stderr, "V");
#ifdef TOAST_SYNC
		int dtime = 0;

		if (buf_entry->flags & BUF_FLAG_CLOCK_VALID) {
			long long clock = (buf_entry->clock * 300 + buf_entry->clock_ext)/27; // MHz
			dtime = bufGetDTime (buf, clock);
		}

		fprintf (stderr, "%d ", dtime);


		dtime /= 1000;

		if (dtime < 0) { // drop frames
//			break;
		} else if (dtime > 1) {
			usleep (dtime-1);
		}
#endif

		if (priv->fd_video >= 0)
			write (priv->fd_video, buf_entry->data, buf_entry->data_len);
		break;
	}
	default:
//		fprintf (stderr, "codec_sw: unhandled stream type\n");
	}

//	ExitMP3 (&mp);

	return NULL;
}


/**
 *
 **/

static int _sw_init (plugin_codec_t *plugin, buf_t *_buf, char *output_path)
{
	struct sw_priv_struct *priv = plugin->priv; 

	priv->buf = _buf;
	
	fprintf (stderr, "Trying to open /tmp/video... ");
	if ((priv->fd_video = open ("/tmp/video", O_WRONLY)) < 0)
		fprintf (stderr, "failed: video stream will be ignored.\n");
	else
		fprintf (stderr, "success.\n");

	fprintf (stderr, "Trying to open /tmp/audio... ");
	if ((priv->fd_audio = open ("/tmp/audio", O_WRONLY)) < 0)
		fprintf (stderr, "failed: audio stream will be ignored.\n");
	else
		fprintf (stderr, "success.\n");

	fprintf (stderr, "Trying to open /tmp/other... ");
	if ((priv->fd_other = open ("/tmp/other", O_WRONLY)) < 0)
		fprintf (stderr, "failed: other streams will be ignored.\n");
	else
		fprintf (stderr, "success.\n");

#ifndef TOAST_SPU
	fprintf (stderr, "Trying to open /tmp/spu... ");
	if ((priv->fd_spu = open ("/tmp/spu", O_WRONLY)) < 0)
		fprintf (stderr, "failed: spu stream will be ignored.\n");
	else
		fprintf (stderr, "success.\n");
#else
	spuInit ();

	bg_image = (u_int *) calloc (1, 720 * 576 * sizeof(u_int));
#endif

	return 0;
}


/**
 *
 **/

static int _sw_set_status (plugin_codec_t *plugin, u_int status)
{
	switch (status) {
	case STATUS_PLAY:
	case STATUS_PAUSE:
	case STATUS_STOP:
		break;
	default:
		fprintf (stderr, "DecoderSetStatus: unhandled status %d\n", status);
		return -1;
	}

	return 0;
}


/**
 *
 **/

static int _sw_set_attributes (plugin_codec_t *plugin, u_int id, void *attr)
{
#ifdef TOAST_SPU
#ifdef DEBUG
	fprintf (stderr, "setting CLUT\n");
#endif
	switch (id) {
	case BUF_SUBPIC: {	
		plugin_codec_attr_spu_t *subpic = attr;
		spuSetCLUT (subpic->clut);
		break;
	}	
	}
#endif
	return 0;
}


/**
 * Initialize Plugin.
 **/

void *plugin_init (char *whoami)
{
	pluginRegister (whoami,
		PLUGIN_ID_CODEC_VIDEO,
		STREAM_ID_MPEG2,
		&codec_sw);

	return &codec_sw;
}


/**
 * Cleanup Plugin.
 **/

void plugin_exit (void)
{
}
