#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>

#include <oms/plugin.h>
#include "input_vcd.h"

static int set_position (int min,int sec);
static int read_toc ();
static int get_track_number (char m, char s, char f);
static int msf_to_lba (char m, char s, char f);
static void next_sector ();

typedef struct {
	char sync	[12],
	header		[4],
	subheader	[8],
	data		[2324],
	spare		[4];
} cdsector;

static vcd_priv_t priv;

static int vcd_open (char *name);
static int vcd_close (void);
static int vcd_read (buf_t *buf, buf_entry_t *buf_entry);
static int vcd_seek (u_long pos);


static plugin_input_t input = {
	vcd_open,
	vcd_close,
	vcd_read,
	vcd_seek,
	CD_FRAMESIZE_RAW,	
	&priv
};


/**
 *
 */

static int vcd_open (char *device)
{
fprintf (stderr, "vcd_open\n");

	if ((priv.fd = open (device, O_RDONLY)) == -1) {
		perror (device);
		return -1;
	}

	if (read_toc ())
		return -1;

	if (vcd_seek (0))
		return -1;

fprintf (stderr, "vcd_open DONE\n");
	return 0;
}


/**
 *
 */

static int vcd_close (void)
{
	close (priv.fd);

	return 0;
}


/**
 * Seek to a specific track
 */

static int vcd_seek (u_long pos)
{
	priv.time.minute = priv.tocent[pos].cdte_addr.msf.minute;
	priv.time.second = priv.tocent[pos].cdte_addr.msf.second;
	priv.time.frame  = priv.tocent[pos].cdte_addr.msf.frame;
	priv.current_track = pos;
	priv.counter=0;

	return 0;
}


/**
 *
 */

static int vcd_read (buf_t *buf, buf_entry_t *buf_entry)
{
	struct cdrom_msf msf;
	int track;

	if (buf_entry->mem_len != CD_FRAMESIZE_RAW)
		return -1;

	msf.cdmsf_min0 = priv.time.minute;
	msf.cdmsf_sec0 = priv.time.second;
	msf.cdmsf_frame0 = priv.time.frame;

	memcpy (buf_entry->mem, &msf, sizeof (msf));

//CDROMREADMODE2
	if (ioctl(priv.fd, CDROMREADRAW, buf_entry->mem)) {
		int t = get_track_number (priv.time.minute,
			priv.time.second,
			priv.time.frame);
		perror("cdromreadraw");
		if (t == priv.tochdr.cdth_trk1)
			return 0;

		vcd_seek (t);
		return vcd_read (buf_entry->mem, buf_entry->mem_len); // try again with next track
	}
	next_sector();

	if (priv.callb.chapter) {
		priv.counter++;
		if (priv.counter % 64 == 0) {
			track=get_track_number (priv.time.minute,
				priv.time.second,
				priv.time.frame)-1;
			if (track != priv.current_track) {
				priv.callb.chapter (priv.callb.chapter_data, track);
				priv.current_track=track;
			}
		}
	}

	return buf_entry->mem_len;
}


/**
 *
 */

static int msf_to_lba (char m, char s, char f)
{
	return (((m * CD_SECS) + s) * CD_FRAMES + f) - CD_MSF_OFFSET;
}


/**
 *
 */

static int get_track_number (char m, char s, char f)
{
	int i;
	int lba2;
	int lba = msf_to_lba(m,s,f);

	for (i=priv.tochdr.cdth_trk0; i <= priv.tochdr.cdth_trk1; i++) {
		lba2 = msf_to_lba(priv.tocent[i-1].cdte_addr.msf.minute,priv.tocent[i-1].cdte_addr.msf.second,priv.tocent[i-1].cdte_addr.msf.frame);
		if (lba2>lba) return i-1;
	}

	return -1;
}


/**
 *
 */

static void next_sector()
{
	priv.time.frame++;

	if (priv.time.frame>=CD_FRAMES) {
		priv.time.frame = 0;
		priv.time.second++;

		if (priv.time.second>=CD_SECS) {
			priv.time.second = 0;
			priv.time.minute++;
		}
	}
}

  
/**
 *
 */

static int read_toc()
{
	int i;

// read TOC header
	if ( ioctl(priv.fd, CDROMREADTOCHDR, &priv.tochdr) == -1 ) {
		fprintf(stderr, "readvcd: ioctl cdromreadtochdr\n");
		return -1;
	}

// read individual tracks
	for (i=priv.tochdr.cdth_trk0; i<=priv.tochdr.cdth_trk1; i++) {
		priv.tocent[i-1].cdte_track = i;
		priv.tocent[i-1].cdte_format = CDROM_MSF;
		if ( ioctl(priv.fd, CDROMREADTOCENTRY, &priv.tocent[i-1]) == -1 ) {
			fprintf(stderr, "readvcd: ioctl cdromreadtocentry\n");
			return -1;
		}
	}

// read the lead-out track
	priv.tocent[priv.tochdr.cdth_trk1].cdte_track = CDROM_LEADOUT;
	priv.tocent[priv.tochdr.cdth_trk1].cdte_format = CDROM_MSF;

	if (ioctl(priv.fd, CDROMREADTOCENTRY, &priv.tocent[priv.tochdr.cdth_trk1]) == -1 ) {
		fprintf(stderr, "readvcd: ioctl cdromreadtocentry\n");
		return -1;
	}

	priv.total_tracks = priv.tochdr.cdth_trk1;

	return 0;
}


#if 0
/**
 *
 */

static int set_position (int min,int sec)
{
	if (min > priv.tocent[priv.tochdr.cdth_trk1].cdte_addr.msf.minute)
		return -1;

	if (min < 0 || sec < 0)
		return -1;

	priv.time.minute=min;
	priv.time.second=sec;
	priv.time.frame=0;

	return 0;
}


// MATS: please check this out, i've no idea if we can map this to
//	seek, or do we have to add an extra entry here?

int VCDSetTime(int time) {
	return set_position(0,time/60,time%60);
}
  
/* TODO: fix track_end */
int VCDSetTrack(int track, int track_end) {
  return set_position(track+1,-1,-1);
}


    
int VCDGetTime() {
  return m*60+s;
      }
      
int VCDGetTotalTime() {
  return  priv.tocent[priv.tochdr.cdth_trk1].cdte_addr.msf.minute*60+priv.tocent[priv.tochdr.cdth_trk1].cdte_addr.msf.second;
      }
      
int VCDGetTrack() {
  return get_track_number(m,s,f)-1;
}
      
int VCDGetTotalTracks() {
  return priv.tochdr.cdth_trk1-1;
      }
      
      

#endif


/**
 * Initialize Plugin.
 **/

void *plugin_init (char *whoami)
{
	pluginRegister (whoami,
                PLUGIN_ID_INPUT,
                0,
                &input);

	return &input;
}


/**
 * Cleanup Plugin.
 **/

void plugin_exit (void)
{
}
