#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#include <oms/oms.h>
#include <oms/plugin.h>
#include <oms/plugin/nav.h>

extern int pluginGetDecapsMagic (plugin_t *plugin, char *filename);

static int fileOpen (plugin_t *plugin, char *path);
static int filePartPlayAutoStop (int title, int chapter, int parts2play);
static nav_tree_t *fileGetInfo (u_int id);
static int fileRegisterCb (u_int id, void (*cb) (void *, int ), void *data);
static int fileUnregisterCb (u_int id);

static struct {
	char *root;
	plugin_t *plugin;
} file_priv;

static plugin_nav_t file_nav = {
	fileOpen,
	NULL,
	filePartPlayAutoStop,
	fileGetInfo,
	fileRegisterCb,
	fileUnregisterCb,
	&file_priv
};



/**
 *
 */

static int fileOpen (plugin_t *plugin, char *path)
{
	file_priv.root = path;
	file_priv.plugin = plugin;

	if (pluginOpen (plugin, PLUGIN_ID_INPUT, DL_INPUT_FILE, path))
		return -1;

	if (pluginGetDecapsMagic (plugin, path))
		return -1;

	PLUGIN_OPS_INPUT(plugin)->blocksize = PLUGIN_OPS_DECAPS(plugin)->blocksize;
	PLUGIN_OPS_INPUT(plugin)->open (path);

	PLUGIN_OPS_DECAPS(plugin)->read = PLUGIN_OPS_INPUT(plugin)->read;

	if (_PLUGIN_OPS_CODEC(plugin))
		PLUGIN_OPS_DECAPS(plugin)->write = _PLUGIN_OPS_CODEC(plugin)->read;

	PLUGIN_OPS_INPUT(plugin)->seek (0);

	PLUGIN_OPS_NAV(plugin) = &file_nav;

        return DVD_PLAYER_OK;
}


/**
 *
 */

static int filePartPlayAutoStop (int title, int chapter, int parts2play)
{
	PLUGIN_OPS_INPUT(file_priv.plugin)->seek (0);

	return DVD_PLAYER_OK;
}


/**
 *
 */

static nav_tree_t *fileGetInfo (u_int id)
{
	nav_tree_t *tree = NULL;

	switch (id) {
	case OMS_INFO_PROGRAM: {
		tree = malloc (sizeof (nav_tree_t));

        	tree->num = 1;
        	tree->title = malloc (sizeof (nav_subtree_t) * tree->num);

		{
			nav_subtree_t *sub_tree = tree->title;
			sub_tree->sub = (nav_tree_data_t *) malloc (sizeof (nav_tree_data_t));
			sub_tree->num = 0;
			sub_tree->name = strdup (file_priv.root);
		}
		break;
	}
	default:
		omsPrint (OMS_PRINT_DEBUG, "unhandled OMS_INFO %s", "tjo");
	}

	return tree;
}


/**
 *
 */

static int fileRegisterCb (u_int id, void (*cb) (void *, int ), void *data)
{
	omsPrint (OMS_PRINT_DEBUG, "UI registered callback");

	switch (id) {
	case CB_CHANGE:
//		dvd_priv.callb.chapter = cb;
//		dvd_priv.callb.chapter_data = data;
		break;
	default:
		return -1;
	}

	return 0;
}


/**
 *
 */

static int fileUnregisterCb (u_int id)
{
	omsPrint (OMS_PRINT_DEBUG, "UI unregistered callback %d", id);

	return 0;
}


/**
 * Initialize plugin
 **/

void *plugin_init (char *whoami)     
{     
	pluginRegister (whoami,
		PLUGIN_ID_DECAPS,
		STREAM_ID_MPEG2,
		&file_nav);

        return &file_nav;     
}     


/**
 * Cleanup clugin
 **/

void plugin_exit (void)
{
	fprintf (stderr, "file plugin_exit\n");
}

