/*  Open Media System - oms
 * 
 *  output device page in configuration dialog
 *
 *  Copyright (C) 2000  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config_store.h"

#include "config_output.h"
#include "config_vo.h"
#include "config_ao.h"
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include <config.h>

static GSList *rbgroup = NULL;

static gchar *strOutputPlugin;
static gchar *strOutputDevice;

typedef struct {
	gchar *name;
	gchar *plugin;
	gchar *device;
} output_info_t;

static output_info_t outputs[] = {
	{
		"Software",
		OMS_PLUGINDIR"/codec_sw.so",
		"x11:oss"
	},
	{
		"Pipes",
		OMS_PLUGINDIR"/codec_pipe.so",
		"/tmp/video"
	},
	{
		"DXR2",
		OMS_PLUGINDIR"/codec_dxr2.so",
		"/dev/dxr2"
	},
	{
		"DXR3",
		OMS_PLUGINDIR"/codec_dxr3.so",
		"/dev/em8300"
	},
	{
		"AVP",
		OMS_PLUGINDIR"/codec_avp.so",
		"/dev/video"
	},
	{
		"TS",
		OMS_PLUGINDIR"/codec_ts.so",
		"/tmp/output.ts"
	}
};


/**
 *
 **/

void config_output_apply ()
{
	char buf[100];

	cfg_set_val_string ("outputplugin", strOutputPlugin);
	// FIXME hack for vo/ao config
	if (strcmp(strOutputPlugin, OMS_PLUGINDIR"/codec_sw.so") == 0)
	{
		snprintf(buf,100,"%s:%s",get_vo_device(),get_ao_device());
		cfg_set_val_string ("outputdevice", buf);
	}
	else
	{
		cfg_set_val_string ("outputdevice", strOutputDevice);
	}
}


/**
 *
 **/

static void output_toggled (GtkButton *button,
		gpointer user_data) 
{
	output_info_t *info;

	g_return_if_fail (button != NULL);
	g_return_if_fail (user_data != NULL);

	info = (output_info_t*)user_data;

	fprintf (stderr, "%s\n", info->name);

	if (GTK_TOGGLE_BUTTON (button)->active)
	{
		strOutputPlugin = info->plugin;
		strOutputDevice = info->device;
	}
}


/**
 *
 **/

static void create_output_option (GtkWidget *vbox,
		output_info_t *info) 
{
	GtkWidget *prbutton;

	prbutton = gtk_radio_button_new_with_label(rbgroup, info->name);
	gtk_widget_show (prbutton);
	gtk_box_pack_start (GTK_BOX (vbox), prbutton, FALSE, FALSE, 5);
	rbgroup = gtk_radio_button_group( GTK_RADIO_BUTTON(prbutton) );
	gtk_signal_connect (GTK_OBJECT (prbutton),
			"toggled",
			GTK_SIGNAL_FUNC(output_toggled),
			(gpointer) info);
	if (strcmp(strOutputPlugin, info->plugin) == 0)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prbutton), TRUE);
	}
}


/**
 *
 **/

GtkWidget *create_output_page ()
{
	GtkWidget *vbox ;
	int i;

	strOutputPlugin = cfg_get_defval_string ("outputplugin",OMS_PLUGINDIR"/codec_sw.so");
	strOutputDevice = cfg_get_defval_string ("outputdevice","x11:oss");

	vbox = gtk_vbox_new (FALSE, 5);
	gtk_widget_show (vbox);

	for (i=0; i<(sizeof(outputs)/sizeof(output_info_t)); i++)
	{
		create_output_option (vbox, &outputs[i]);
	}

	return vbox;
}
