/*  Open Media System - oms
 * 
 *  video out page in configuration dialog
 *
 *  Copyright (C) 2000  Thomas Mirlacher, Guenter Bartsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config_store.h"

#include "config_vo.h"
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include <config.h>

#include <video_out.h>

static GSList *rbgroup = NULL;

static gchar     defaultOutputDevice[100];
static gchar     *strOutputDevice;


/**
 *
 **/

const char* get_vo_device (void)
{
	return strOutputDevice;
}


/**
 *
 **/

static void vo_toggled (GtkButton *button,
			const gpointer user_data) 
{
	const vo_info_t *info;

	g_return_if_fail (button != NULL);
	g_return_if_fail (user_data != NULL);

	info = (vo_info_t*)user_data;

	if (GTK_TOGGLE_BUTTON (button)->active) {
		strOutputDevice = info->short_name;
	}
}

/**
 * 
 **/

static void create_display_option (GtkWidget *vbox,
		const vo_info_t *info,
		const char *current_driver)
{
	GtkWidget *button;

	button = gtk_radio_button_new_with_label (rbgroup, info->name);
	gtk_widget_show (button);
	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 5);
	rbgroup = gtk_radio_button_group (GTK_RADIO_BUTTON(button));
	gtk_signal_connect (GTK_OBJECT (button),
			"toggled",
			GTK_SIGNAL_FUNC(vo_toggled),
			(gpointer) info);
	if (strcmp(current_driver, info->short_name) == 0)
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), TRUE);
	}
}

/**
 *
 **/

GtkWidget *create_vo_page ()
{
	GtkWidget *vbox;
	int i;
	char *tmp, *vo, *ao;
	char buf[100];

	vbox = gtk_vbox_new (FALSE, 5);

	// FIXME yuck!
	tmp = cfg_get_defval_string ("outputdevice","x11:oss");
        if ( strlen(tmp) > 0 ) {
                strncpy(buf,tmp,sizeof(buf));
                tmp = buf;
                vo = strtok(tmp, ":, ");
                ao = strtok(NULL, ":, ");
                strcpy(defaultOutputDevice,vo);
        }

        strOutputDevice = defaultOutputDevice;

	for (i=0 ; video_out_drivers[i] != NULL; i++) {
		const vo_info_t* info = video_out_drivers[i]->get_info();
		create_display_option (vbox, info, strOutputDevice);
	}

	gtk_widget_show (vbox);
			
	return vbox;
}


