%{
#include <string.h>
#include "y.tab.h"

#ifdef yywrap
#undef yywrap
#endif

void yyerror (char *msg);
static int check_keyword (char *string);

int yylineno = 1;
%}


/**
 * state definitions
 */

%s NAME NORMAL


/**
 * pattern deefinitions
 */

ws		[\t ]+
nl		\n
comment		#.*
integer		[0-9]+
filename	[a-zA-Z0-9\.\\\/_]+
keyword		[a-zA-Z0-9]+
bool_true	[Tt][Rr][Uu][Ee]
bool_false	[Ff][Aa][Ll][Ss][Ee]

/**
 * and now, here it comes: we're proud to present: THE RULES!
 */

BEGIN KEY;
%%

<INITIAL>{keyword}	{	int ret;
				if ((ret = check_keyword (yytext)) >= 0) {
			  		BEGIN NAME;
					return ret;
				} else {
					printf ("ERROR (parsing skin config file): unknown identifier \"%s\"in line %d\n", yytext, yylineno);
					return -1;
				}
			}

{bool_true}		{	yylval.integer = 1;
			 	return BOOLEAN;
			}
{bool_false}		{	yylval.integer = 0;
			 	return BOOLEAN;
			}

<NAME>{filename}	{	yylval.string = strdup (yytext);
			 	BEGIN NORMAL;
			 	return FILENAME;
			}
{integer}		{	yylval.integer = atoi (yytext);
			 	return INTEGER;
			}

{comment}		;	// ignore comment
{nl}			{	BEGIN INITIAL;
			 	yylineno++;
			}

.			;	// ignore everything else
<<EOF>>			{	return 0;
			}

%%


/**
 * tell lexer to stop at end of current file
 */

int yywrap (void) {
	return 1;
}


/**
 * out customized error function
 */

void yyerror (char *msg)
{
	printf ("%d: %s\n", yylineno, msg);
}


/**
 * semi-dynamic check for keywords
 */

#define __NEED_SKIN_KEYWORD
#include "skin_keyword.h"

static int check_keyword (char *string)
{
	struct keyword *ptr = keywords;

	while (ptr->name) {
		if (!strcasecmp (ptr->name, string)) {
			yylval.integer = ptr->value; 
			return ptr->type;
		} else
			ptr ++;
	}

	return -1;
}
