%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <config.h>

#ifdef HAVE_IMLIB2
#include <Imlib2.h>
#else
#include <gdk_imlib.h>
#endif

#include "skin.h"

extern void yyerror (char *msg);
static void loadImg (gchar *strName_, GdkPixmap **pix, GdkBitmap **mask);
%}

%union {
	int integer;
	char *string;
}

%token T_PIXMAP T_TEXT T_STATUSBUTTON T_PUSHBUTTON
%token FILENAME BOOLEAN INTEGER
%%

config:		config_line
	|	config_line config
		;
		
config_line:	pixmap
	|	text
	|	pushbutton
	|	statusbutton
		;

pixmap:		T_PIXMAP FILENAME 
		{
			loadImg ($<string>2,
				&skin[$<integer>1].pixmap,
				&skin[$<integer>1].mask);
			free ($<string>2);
		};

text:		T_TEXT FILENAME INTEGER INTEGER INTEGER 
		{
			loadImg ($<string>2,
				&skin[$<integer>1].pixmap,
				&skin[$<integer>1].mask);
			free ($<string>2);
			skin[$<integer>1].x = $<integer>3;
			skin[$<integer>1].y = $<integer>4; 
			skin[$<integer>1].nMax = $<integer>5;
		};

pushbutton:	T_PUSHBUTTON FILENAME BOOLEAN INTEGER INTEGER 
		{
			loadImg ($<string>2,
				&skin[$<integer>1].pixmap,
				&skin[$<integer>1].mask);
			free ($<string>2);
			skin[$<integer>1].bPrelight = $<integer>3;
			skin[$<integer>1].bStatus	= 0;
			skin[$<integer>1].x = $<integer>4;
			skin[$<integer>1].y = $<integer>5;
		};

statusbutton:	T_STATUSBUTTON FILENAME BOOLEAN BOOLEAN INTEGER INTEGER 
		{
			loadImg ($<string>2,
				&skin[$<integer>1].pixmap,
				&skin[$<integer>1].mask);
			free ($<string>2);
			skin[$<integer>1].bPrelight = $<integer>3;
			skin[$<integer>1].bStatus = $<integer>4;
			skin[$<integer>1].x = $<integer>5;
			skin[$<integer>1].y = $<integer>6;
		};

%%

extern int yyparse ();
extern FILE *yyin;


/**
 *
 */

skin_t *get_skin (int i)
{
        return &skin[i];
}


/**
 *
 */

GdkPixmap *get_skin_pixmap (int i)
{
	skin_t *pSkin = get_skin (i);

	if (pSkin)
		return pSkin->pixmap;
	else
		return NULL;
}

static char *skindir;


/**
 *
 */

int load_skin (char *name)
{
	char *skindata;

	skindir = (char *) malloc (strlen (OMS_SKINDIR) + strlen (name) + 3);
	sprintf (skindir, "%s/%s", OMS_SKINDIR, name);

	skindata = (char *) malloc (strlen (skindir) + 10); // 10 = /skindata
	sprintf (skindata, "%s/skindata", skindir);

#ifdef DEBUG
//	fprintf (stderr, "Opening skindir: %s...\n", skindir);
//	fprintf (stderr, "Opening skindata: %s...\n", skindata);
#endif

	if ((yyin = fopen (skindata, "r")) < 0)
		return -1;
	free (skindata);

	yyparse ();

	fclose (yyin);

	free (skindir);

	return 0;
}


/**
 *
 */

static void loadImg (gchar *strName_, GdkPixmap **pix, GdkBitmap **mask)
{
       	gchar filename[1024];
#ifdef HAVE_IMLIB2
	Imlib_Image *pImage;
#else
	GdkImlibImage *pImage;
#endif
   
	snprintf (filename, 1024, "%s/%s",skindir,strName_);
 
#ifdef HAVE_IMLIB2 
	if ((pImage = imlib_load_image (filename))) {
		imlib_render_pixmaps_for_whole_image (pix, masp, FALSE);
		imlib_free_image_and_decache ();
#else
	if ((pImage = gdk_imlib_load_image (filename))) {
		gdk_imlib_render (pImage,
			pImage->rgb_width,
			pImage->rgb_height);

		*pix = gdk_imlib_move_image (pImage);
		*mask = gdk_imlib_move_mask (pImage);
		gdk_imlib_destroy_image (pImage);
#endif
	} else 
		fprintf (stderr, "Error loading image %s\n", filename);
}
